function varargout = CODIS(varargin)
%This GUI is a tool to compute the Paternity Index in a lawsuit for paternity
%attribuition or exclusion. It is possible to use the Applied Biosystems or
%Promega forensic kits. The GUI can be use in presence or in absence of the
%mother.
%Example: run CODIS and select the DEMO menu
%
%           Created by Giuseppe Cardillo
%           giuseppe.cardillo-edta@poste.it
%
% To cite this file, this would be an appropriate format:
% Cardillo G. (2009). CODIS: COmbined DNA Index System. A GUI for forensic paternity lawsuit.
% http://www.mathworks.com/matlabcentral/fileexchange/24111

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @CODIS_OpeningFcn, ...
    'gui_OutputFcn',  @CODIS_OutputFcn, ...
    'gui_LayoutFcn',  [], ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

% --- Executes just before CODIS is made visible.
function CODIS_OpeningFcn(hObject, ~, handles, varargin)
handles.output = hObject;
guidata(hObject, handles);
%M is the matrix of the Mother's data
%C is the matrix of the Child's data
%F is the matrix of the Alleged Father's data
%IC is the Inbreeding Coefficient
%CODISpop is a cell matric containing several informations as the alleles
%frequencies and the mutations rates.
global M C F Hpstring Hdstring FHp FHd Tutflag 
global CODISRootFolder fname KIT Kitsorted
global CODISpop Parentage %#ok<NUSED>
M=ones(18,2); C=M; F=M; FHp=1; FHd=1;
if exist(strcat(CODISRootFolder,'CODISpwd.mat'),'file')==0
    CODISpwd=pwd; 
    save CODISpwd.mat CODISpwd; 
    path(CODISpwd,path);
else
    load CODISpwd.mat CODISpwd
    cd(CODISpwd)
end
%the root directory of this function
CODISRootFolder = strcat(pwd,filesep); 
%Check if system folders are present. On the contrary create them
if exist(strcat(CODISRootFolder,'data'),'dir')==0
    mkdir(CODISRootFolder,'data')
end
if exist(strcat(CODISRootFolder,'stats'),'dir')==0
    mkdir(CODISRootFolder,'stats')
end
if exist(strcat(CODISRootFolder,'reports'),'dir')==0
    mkdir(CODISRootFolder,'reports')
end
fname='';
%Select the proper datasets for population and kit
pathname=strcat(CODISRootFolder,'alleles',filesep);
cd(pathname)
eval(['load(''' int2str(get(handles.Populationpopupmenu,'Value')) '.mat'',''CODISpop'')'])
load('kit.mat','KIT')
cd(CODISRootFolder);
Kitsorted=sort(KIT.Kitname);
set(handles.Kitpopupmenu,'String',Kitsorted);
popdataset(handles)
Hpstring=get(handles.Hppopupmenu,'String');
Hdstring=get(handles.Hdpopupmenu,'String');
Tutflag=0; Parentage=1;

% --- Outputs from this function are returned to the command line.
function varargout = CODIS_OutputFcn(~, ~, handles)
varargout{1} = handles.output;

%These functions were automatically generated. I think they are useless, but
%they cannot be erased...
% --- Executes during object creation, after setting all properties.
function Motherlabel1_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel1_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel2_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel2_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel3_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel3_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel4_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel4_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel5_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel5_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel6_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel6_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel7_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel7_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel8_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel8_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel9_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel9_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel10_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel10_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel11_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel11_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel12_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel12_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel13_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel13_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel14_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel14_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel15_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel15_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel16_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel16_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel17_1_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.
function Motherlabel17_2_CreateFcn(~, ~, ~) %#ok<DEFNU>
% --- Executes during object creation, after setting all properties.

function Childlabel1_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel1_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel2_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel2_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel3_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel3_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel4_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel4_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel5_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel5_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel6_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel6_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel7_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel7_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel8_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel8_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel9_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel9_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel10_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel10_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel11_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel11_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel12_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel12_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel13_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel13_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel14_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel14_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel15_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel15_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel16_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel16_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel17_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function Childlabel17_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes during object creation, after setting all properties.
function Hdpopupmenu_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes during object creation, after setting all properties.
function Hppopupmenu_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

%Use the Motherpopupmenu to setup the data matrix
% --- Executes on selection change in Motherpopupmenu1_1.
function Motherpopupmenu1_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell cell
global M
M(1,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu1_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu1_2.
function Motherpopupmenu1_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(1,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu1_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu2_1.
function Motherpopupmenu2_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(2,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu2_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu2_2.
function Motherpopupmenu2_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(2,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu2_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu3_1.
function Motherpopupmenu3_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(3,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu3_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu3_2.
function Motherpopupmenu3_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(3,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu3_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu4_1.
function Motherpopupmenu4_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(4,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu4_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu4_2.
function Motherpopupmenu4_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(4,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu4_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu5_1.
function Motherpopupmenu5_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(5,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu5_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu5_2.
function Motherpopupmenu5_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(5,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu5_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu6_1.
function Motherpopupmenu6_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(6,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu6_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu6_2.
function Motherpopupmenu6_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(6,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu6_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu7_1.
function Motherpopupmenu7_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(7,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu7_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu7_2.
function Motherpopupmenu7_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(7,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu7_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu8_1.
function Motherpopupmenu8_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(8,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu8_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu8_2.
function Motherpopupmenu8_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(8,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu8_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu9_1.
function Motherpopupmenu9_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(9,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu9_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu9_2.
function Motherpopupmenu9_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(9,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu9_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu10_1.
function Motherpopupmenu10_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(10,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu10_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu10_2.
function Motherpopupmenu10_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(10,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu10_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu11_1.
function Motherpopupmenu11_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(11,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu11_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu11_2.
function Motherpopupmenu11_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(11,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu11_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu12_1.
function Motherpopupmenu12_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(12,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu12_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu12_2.
function Motherpopupmenu12_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(12,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu12_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu13_1.
function Motherpopupmenu13_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(13,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu13_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu13_2.
function Motherpopupmenu13_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(13,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu13_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu14_1.
function Motherpopupmenu14_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(14,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu14_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu14_2.
function Motherpopupmenu14_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(14,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu14_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu15_1.
function Motherpopupmenu15_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(15,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu15_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu15_2.
function Motherpopupmenu15_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(15,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu15_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu16_1.
function Motherpopupmenu16_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(16,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu16_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu16_2.
function Motherpopupmenu16_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(16,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu16_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu17_1.
function Motherpopupmenu17_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(17,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu17_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu17_2.
function Motherpopupmenu17_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(17,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu17_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu18_1.
function Motherpopupmenu18_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(18,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu18_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Motherpopupmenu18_2.
function Motherpopupmenu18_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global M
M(18,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Motherpopupmenu18_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

%Use the Childpopupmenu to setup the data matrix
% --- Executes on selection change in Childpopupmenu1_1.
function Childpopupmenu1_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(1,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu1_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu1_2.
function Childpopupmenu1_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(1,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu1_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu2_1.
function Childpopupmenu2_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(2,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu2_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu2_2.
function Childpopupmenu2_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(2,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu2_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu3_1.
function Childpopupmenu3_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(3,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu3_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu3_2.
function Childpopupmenu3_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(3,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu3_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu4_1.
function Childpopupmenu4_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(4,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu4_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu4_2.
function Childpopupmenu4_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(4,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu4_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu5_1.
function Childpopupmenu5_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(5,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu5_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu5_2.
function Childpopupmenu5_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(5,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu5_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu6_1.
function Childpopupmenu6_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(6,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu6_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu6_2.
function Childpopupmenu6_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(6,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu6_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu7_1.
function Childpopupmenu7_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(7,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu7_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu7_2.
function Childpopupmenu7_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(7,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu7_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu8_1.
function Childpopupmenu8_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(8,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu8_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu8_2.
function Childpopupmenu8_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(8,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu8_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu9_1.
function Childpopupmenu9_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(9,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu9_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu9_2.
function Childpopupmenu9_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(9,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu9_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu10_1.
function Childpopupmenu10_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(10,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu10_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu10_2.
function Childpopupmenu10_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(10,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu10_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu11_1.
function Childpopupmenu11_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(11,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu11_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu11_2.
function Childpopupmenu11_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(11,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu11_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu12_1.
function Childpopupmenu12_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(12,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu12_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu12_2.
function Childpopupmenu12_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(12,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu12_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu13_1.
function Childpopupmenu13_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(13,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu13_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu13_2.
function Childpopupmenu13_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(13,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu13_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu14_1.
function Childpopupmenu14_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(14,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu14_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu14_2.
function Childpopupmenu14_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(14,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu14_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu15_1.
function Childpopupmenu15_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(15,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu15_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu15_2.
function Childpopupmenu15_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(15,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu15_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu16_1.
function Childpopupmenu16_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(16,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu16_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu16_2.
function Childpopupmenu16_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(16,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu16_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu17_1.
function Childpopupmenu17_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(17,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu17_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu17_2.
function Childpopupmenu17_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(17,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu17_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu18_1.
function Childpopupmenu18_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(18,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu18_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Childpopupmenu18_2.
function Childpopupmenu18_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global C
C(18,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Childpopupmenu18_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


%Use the Fatherpopupmenu to setup the data matrix
% --- Executes on selection change in Fatherpopupmenu1_1.
function Fatherpopupmenu1_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(1,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu1_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu1_2.
function Fatherpopupmenu1_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(1,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu1_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu2_1.
function Fatherpopupmenu2_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(2,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu2_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu2_2.
function Fatherpopupmenu2_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(2,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu2_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu3_1.
function Fatherpopupmenu3_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(3,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu3_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu3_2.
function Fatherpopupmenu3_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(3,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu3_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu4_1.
function Fatherpopupmenu4_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(4,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu4_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu4_2.
function Fatherpopupmenu4_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(4,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu4_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu5_1.
function Fatherpopupmenu5_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(5,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu5_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu5_2.
function Fatherpopupmenu5_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(5,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu5_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu6_1.
function Fatherpopupmenu6_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(6,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu6_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu6_2.
function Fatherpopupmenu6_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(6,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu6_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu7_1.
function Fatherpopupmenu7_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(7,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu7_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu7_2.
function Fatherpopupmenu7_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(7,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu7_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu8_1.
function Fatherpopupmenu8_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(8,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu8_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu8_2.
function Fatherpopupmenu8_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(8,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu8_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu9_1.
function Fatherpopupmenu9_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(9,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu9_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu9_2.
function Fatherpopupmenu9_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(9,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu9_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu10_1.
function Fatherpopupmenu10_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(10,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu10_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu10_2.
function Fatherpopupmenu10_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(10,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu10_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu11_1.
function Fatherpopupmenu11_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(11,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu11_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu11_2.
function Fatherpopupmenu11_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(11,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu11_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu12_1.
function Fatherpopupmenu12_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(12,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu12_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu12_2.
function Fatherpopupmenu12_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(12,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu12_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu13_1.
function Fatherpopupmenu13_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(13,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu13_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu13_2.
function Fatherpopupmenu13_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(13,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu13_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu14_1.
function Fatherpopupmenu14_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(14,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu14_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu14_2.
function Fatherpopupmenu14_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(14,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu14_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu15_1.
function Fatherpopupmenu15_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(15,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu15_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu15_2.
function Fatherpopupmenu15_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(15,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu15_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu16_1.
function Fatherpopupmenu16_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(16,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu16_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu16_2.
function Fatherpopupmenu16_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(16,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu16_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu17_1.
function Fatherpopupmenu17_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(17,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu17_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu17_2.
function Fatherpopupmenu17_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(17,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu17_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu18_1.
function Fatherpopupmenu18_1_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(18,1)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu18_1_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Fatherpopupmenu18_2.
function Fatherpopupmenu18_2_Callback(hObject, ~, ~) %#ok<DEFNU>
%Assign the selected label to the proper matrix cell
global F
F(18,2)=get(hObject,'Value');

% --- Executes during object creation, after setting all properties.
function Fatherpopupmenu18_2_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Populationpopupmenu.
function Populationpopupmenu_Callback(~, ~, handles) %#ok<DEFNU>
%Set the population dataset
popdataset(handles)

% --- Executes during object creation, after setting all properties.
function Populationpopupmenu_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Kitpopupmenu.
function Kitpopupmenu_Callback(~, ~, handles) %#ok<DEFNU>
%Set the used kit dataset
popdataset(handles)

% --- Executes during object creation, after setting all properties.
function Kitpopupmenu_CreateFcn(hObject, ~, ~) %#ok<DEFNU>
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in Hppopupmenu.
function Hppopupmenu_Callback(hObject, ~, handles) %#ok<DEFNU>
global FHp Hdstring
FHp=get(hObject,'Value');
if isequal(get(handles.Incestno,'Checked'),'on')==1 && isequal(get(handles.Directparentage,'Checked'),'on')==1
    switch FHp
        case 1
            set(handles.Hdpopupmenu,'String',Hdstring,'Value',1)
            set(handles.uipanel1,'Visible','on')
        case 2
            set(handles.Hdpopupmenu,'String',{'Unrelated','Half Sibs'},'Value',1)
            set(handles.uipanel1,'Visible','off')
        case 3
            set(handles.Hdpopupmenu,'String',{'Unrelated','Full Sibs'},'Value',1)
            set(handles.uipanel1,'Visible','off')
        otherwise
            set(handles.Hdpopupmenu,'String',{'Unrelated'},'Value',1)
            set(handles.uipanel1,'Visible','on')
    end
end

% --- Executes on selection change in Hdpopupmenu.
function Hdpopupmenu_Callback(hObject, ~, ~) %#ok<DEFNU>
global FHd
FHd=get(hObject,'Value');

% --- Executes on button press in pushbutton1 (COMPUTE! when there is not an incestuous relationship).
function pushbutton1_Callback(~, ~, handles) %#ok<DEFNU>
global M C F matM matF pmatF Amb RMNE EP MP CPI CMP FHp FHd PI Tutflag %#ok<NUSED>
global CODISRootFolder
pathname=strcat(CODISRootFolder,'subroutines',filesep);
Rb2=isequal(get(handles.Directparentage,'Checked'),'on');
cd(pathname)
%switch off this push button...
set(handles.pushbutton1,'Visible','off')
%... and make visible the "reset" button and the report menu
set(handles.pushbutton3,'Visible','on')
set(handles.loadmenu,'Enable','off')
set(handles.Savemenu,'Enable','on')
set(handles.Demomenu,'Enable','off')
set(handles.STRinfomenu,'Enable','off')
set(handles.Optionsmenu,'Enable','off')
set(handles.Reportmenu1,'Visible','on')
%for compatibility and to make easier the implementation of the routine, set as
%0 all the cells that were unselected (in this case the label of the poupmenu
%was 1, remember?)
M(M==1)=0; C(C==1)=0; F(F==1)=0;

if ~Tutflag
    switch FHp
        case 1
            set(handles.uipanel3,'Title','Alleged Father')
        case 2
            set(handles.uipanel3,'Title','Alleged Full Sib')
        case 3
            set(handles.uipanel3,'Title','Alleged Half Sib')
        case 4
            set(handles.uipanel3,'Title','Alleged Grandparent')
        case 5
            set(handles.uipanel3,'Title','Alleged Uncle/Aunt (Full)')
        case 6
            set(handles.uipanel3,'Title','Alleged Uncle/Aunt (Half)')
        case 7
            set(handles.uipanel3,'Title','Alleged Great-Uncle/Aunt (1st cousin)')
    end
end
if Rb2==1
    if FHp==2 %Full Sibs
        if FHd==1 %vs unrelated
            PI=CODISSIBS(handles,1/4,1/4,1/4);
        else
            %First compute Full-sibs vs Unrelated
            LR1=CODISSIBS(handles,1/4,1/4,1/4); 
            %Then compute Half-sibs vs Unrelated
            LR2=CODISSIBS(handles,1/2,1/4,0);
            %Finally divide them each other.
            PI=LR1./LR2;
            PI(isnan(PI))=0;
        end
    elseif FHp==3 %Half-Sibs
        if FHd==1 %vs unrelated
            PI=CODISSIBS(handles,1/2,1/4,0);
        else
            %First compute Full-sibs vs Unrelated
            LR1=CODISSIBS(handles,1/2,1/4,0); 
            %Then compute Half-sibs vs Unrelated
            LR2=CODISSIBS(handles,1/4,1/4,1/4);
            %Finally divide them each other.
            PI=LR1./LR2;
            PI(isnan(PI))=0;
        end
    else
        if FHp==1 %Father-Child
            if FHd==1 %vs Unrelated
                %In this case we must compute only PI
                PI=CODIS3(handles,0.5);
            else %vs a relative of the alleged father
                %First compute Father-Child vs Unrelated
                LR1=CODIS3(handles,0.5);
                %Then compute Relative vs Unrelated
                ar=[1/4 1/4 1/8 1/8];
                LR2=CODIS3(ar(FHd-1));
                %Finally divide them each other.
                PI=LR1./LR2;
                PI(isnan(PI))=0;
            end
        else %FHp>=4
            ar=[1/4 1/4 1/8 1/16];
            %In this case we must compute only AI
            PI=CODIS3(handles,ar(FHp-3));
        end
    end
else
    PI=CODIS3REV(handles);
end
cd(CODISRootFolder)

%set the gray background color for all three panels
for J=1:3
    eval(['set(handles.uipanel' int2str(J) ',''BackgroundColor'',[0.702 0.702 0.702])'])
end

for J=1:18 %for each possible STR..
    %switch off the popupmenus
    eval(['set(handles.Motherpopupmenu' int2str(J) '_1,''Visible'',''off'')'])
    eval(['set(handles.Motherpopupmenu' int2str(J) '_2,''Visible'',''off'')'])
    eval(['set(handles.Childpopupmenu' int2str(J) '_1,''Visible'',''off'')'])
    eval(['set(handles.Childpopupmenu' int2str(J) '_2,''Visible'',''off'')'])
    eval(['set(handles.Fatherpopupmenu' int2str(J) '_1,''Visible'',''off'')'])
    eval(['set(handles.Fatherpopupmenu' int2str(J) '_2,''Visible'',''off'')'])
    %If there are not informations about child and alleged father
    if ~all(F(J,:)) || ~all(C(J,:))
        %switch off all the accessory labels
        eval(['set(handles.text' int2str(J) ',''Visible'',''off'')'])
        eval(['set(handles.Commlabel' int2str(J) ',''Visible'',''off'')'])
        eval(['set(handles.PIlabel' int2str(J) ',''Visible'',''off'')'])
        eval(['set(handles.RMNElabel' int2str(J) ',''Visible'',''off'')'])
    else %There are informations about child and alleged father
        %Set the a,b,c variables to use if Amb(J)=1
        %a=possible alleles for the J-th STR
        %b=position of label1: if Amb(J)=1 move it to the centre of the panel
        %c=concatenation of the observed alleles
        eval(['a=get(handles.Motherpopupmenu' int2str(J) '_1,''String'');']);
        eval(['b=get(handles.Childlabel' int2str(J) '_1,''Position'');']); b(1)=0.3; b(3)=0.4;
        c=strcat(a(C(J,1)),'|',a(C(J,2))); 

        if all(M(J,:)) %If there are the mother's data
            if ~matM(J)
                %If there isn't a match berween mother and child (a very rare case), set the
                %mothers labels with a red background (to highlight the mismatch)
                eval(['set(handles.Motherlabel' int2str(J) '_1,''BackgroundColor'',''red'',''TooltipString'',''Red'',''ForegroundColor'',''white'',''FontWeight'',''bold'',''String'',a(M(J,1)),''Visible'',''on'')'])
                eval(['set(handles.Motherlabel' int2str(J) '_2,''BackgroundColor'',''red'',''TooltipString'',''Red'',''ForegroundColor'',''white'',''FontWeight'',''bold'',''String'',a(M(J,2)),''Visible'',''on'')'])
                %In this case, CODIS3 recalls CODIS2, using only the child's and the alleged
                %father's data. Set the child's label background, fontweight and position ond
                %the basisi of the Amb(J) status.
                if ~Amb(J)
                    eval(['set(handles.Childlabel' int2str(J) '_1,''BackgroundColor'',''yellow'',''TooltipString'',''Yellow'',''FontWeight'',''bold'',''String'',a(C(J,1)),''Visible'',''on'')'])
                else
                    eval(['set(handles.Childlabel' int2str(J) '_1,''Position'',b,''BackgroundColor'',''magenta'',''TooltipString'',''Fuchsia'',''String'',c,''Visible'',''on'')'])
                end
                %There is a match between the mother and the child. In this case,
                %set the child's label background, fontweight and position only if Amb(J)=1
            else
                if ~Amb(J)
                    eval(['set(handles.Motherlabel' int2str(J) '_1,''TooltipString'',''White'',''String'',a(M(J,1)),''Visible'',''on'')'])
                    eval(['set(handles.Motherlabel' int2str(J) '_2,''TooltipString'',''Lightpink'',''String'',a(M(J,2)),''Visible'',''on'')'])
                    eval(['set(handles.Childlabel' int2str(J) '_1,''TooltipString'',''Lightpink'',''String'',a(C(J,1)),''Visible'',''on'')'])
                    eval(['set(handles.Childlabel' int2str(J) '_2,''String'',a(C(J,2)),''Visible'',''on'')'])
                else
                    eval(['set(handles.Motherlabel' int2str(J) '_1,''Position'',b,''BackgroundColor'',''magenta'',''TooltipString'',''Fuchsia'',''String'',c,''Visible'',''on'')'])
                    eval(['set(handles.Childlabel' int2str(J) '_1,''Position'',b,''BackgroundColor'',''magenta'',''TooltipString'',''Fuchsia'',''String'',c,''Visible'',''on'')'])
                end
            end

            %Now check the match between the child and the alleged father
            if ~matF(J) %there is not a match...
                if get(handles.Hppopupmenu,'Value')==1
                    if ~pmatF(J)%...and ther is not a probable match, too
                        %Set the comment label
                        eval(['set(handles.Commlabel' int2str(J) ',''ForegroundColor'',''red'',''String'',''mismatch'')'])
                        %set the alleged father's labels with a red background (to highlight the mismatch)
                        eval(['set(handles.Fatherlabel' int2str(J) '_1,''BackgroundColor'',''red'',''TooltipString'',''Red'',''ForegroundColor'',''white'',''FontWeight'',''bold'',''String'',a(F(J,1)),''Visible'',''on'')'])
                        eval(['set(handles.Fatherlabel' int2str(J) '_2,''BackgroundColor'',''red'',''TooltipString'',''Red'',''ForegroundColor'',''white'',''FontWeight'',''bold'',''String'',a(F(J,2)),''Visible'',''on'')'])
                        if ~Amb(J)
                            eval(['set(handles.Childlabel' int2str(J) '_2,''BackgroundColor'',''yellow'',''TooltipString'',''Yellow'',''FontWeight'',''bold'')'])
                        end
                    else %...but there is a possible match!
                        %Set the comment label
                        eval(['set(handles.Commlabel' int2str(J) ',''BackgroundColor'',[1 0.694 0.392],''TooltipString'',''Darkorange'',''ForegroundColor'',''black'',''String'',''match?'')'])
                        %set all the labels with an orange background
                        eval(['set(handles.Fatherlabel' int2str(J) '_1,''BackgroundColor'',[1 0.694 0.392],''TooltipString'',''Darkorange'',''ForegroundColor'',''black'',''FontWeight'',''bold'',''String'',a(F(J,1)),''Visible'',''on'')'])
                        eval(['set(handles.Fatherlabel' int2str(J) '_2,''BackgroundColor'',[1 0.694 0.392],''TooltipString'',''Darkorange'',''ForegroundColor'',''black'',''FontWeight'',''bold'',''String'',a(F(J,2)),''Visible'',''on'')'])
                        eval(['set(handles.Motherlabel' int2str(J) '_1,''BackgroundColor'',[1 0.694 0.392],''TooltipString'',''Darkorange'',''ForegroundColor'',''black'',''FontWeight'',''bold'')'])
                        eval(['set(handles.Motherlabel' int2str(J) '_2,''BackgroundColor'',[1 0.694 0.392],''TooltipString'',''Darkorange'',''ForegroundColor'',''black'',''FontWeight'',''bold'')'])
                        eval(['set(handles.Childlabel' int2str(J) '_1,''BackgroundColor'',[1 0.694 0.392],''TooltipString'',''Darkorange'',''ForegroundColor'',''black'',''FontWeight'',''bold'')'])
                        eval(['set(handles.Childlabel' int2str(J) '_2,''BackgroundColor'',[1 0.694 0.392],''TooltipString'',''Darkorange'',''ForegroundColor'',''black'',''FontWeight'',''bold'')'])
                    end
                else
                    eval(['set(handles.Commlabel' int2str(J) ',''ForegroundColor'',''red'',''String'',''no match'')'])
                    eval(['set(handles.Childlabel' int2str(J) '_2,''BackgroundColor'',''yellow'',''TooltipString'',''Yellow'',''ForegroundColor'',''black'',''FontWeight'',''bold'')'])
                    eval(['set(handles.Fatherlabel' int2str(J) '_1,''BackgroundColor'',''yellow'',''TooltipString'',''Yellow'',''ForegroundColor'',''black'',''FontWeight'',''bold'',''String'',a(F(J,1)),''Visible'',''on'')'])
                    eval(['set(handles.Fatherlabel' int2str(J) '_2,''BackgroundColor'',''yellow'',''TooltipString'',''Yellow'',''ForegroundColor'',''black'',''FontWeight'',''bold'',''String'',a(F(J,2)),''Visible'',''on'')'])
                end
            else %there is a match
                %Set the comment label
                eval(['set(handles.Commlabel' int2str(J) ',''ForegroundColor'',''green'',''String'',''match'')'])
                %check if Amb(J)=1 and set the labels properly
                if ~Amb(J)
                    eval(['set(handles.Childlabel' int2str(J) '_2,''TooltipString'',''DeepSkyBlue'',''String'',a(C(J,2)),''Visible'',''on'')'])
                    eval(['set(handles.Fatherlabel' int2str(J) '_1,''TooltipString'',''DeepSkyBlue'',''String'',a(F(J,1)),''Visible'',''on'')'])
                    eval(['set(handles.Fatherlabel' int2str(J) '_2,''TooltipString'',''White'',''String'',a(F(J,2)),''Visible'',''on'')'])
                else
                    eval(['set(handles.Fatherlabel' int2str(J) '_1,''Position'',b,''BackgroundColor'',''magenta'',''TooltipString'',''Fuchsia'',''String'',c,''Visible'',''on'')'])
                end
            end
            %in this case, the mother is unavailable, but the settings are the same of the
            %previous section
        else
            if ~matF(J)
                eval(['set(handles.Commlabel' int2str(J) ',''ForegroundColor'',''red'',''String'',''no match'')'])
                if get(handles.Hppopupmenu,'Value')==1
                    eval(['set(handles.Childlabel' int2str(J) '_1,''Position'',b,''BackgroundColor'',''yellow'',''TooltipString'',''Yellow'',''FontWeight'',''bold'',''String'',c,''Visible'',''on'')'])
                else
                    eval(['set(handles.Childlabel' int2str(J) '_1,''BackgroundColor'',''yellow'',''TooltipString'',''Yellow'',''ForegroundColor'',''black'',''FontWeight'',''bold'',''String'',a(C(J,1)),''Visible'',''on'')'])
                    eval(['set(handles.Childlabel' int2str(J) '_2,''BackgroundColor'',''yellow'',''TooltipString'',''Yellow'',''ForegroundColor'',''black'',''FontWeight'',''bold'',''String'',a(C(J,2)),''Visible'',''on'')'])
                end
                if get(handles.Hppopupmenu,'Value')==1
                    eval(['set(handles.Fatherlabel' int2str(J) '_1,''BackgroundColor'',''red'',''TooltipString'',''Red'',''ForegroundColor'',''white'',''FontWeight'',''bold'',''String'',a(F(J,1)),''Visible'',''on'')'])
                    eval(['set(handles.Fatherlabel' int2str(J) '_2,''BackgroundColor'',''red'',''TooltipString'',''Red'',''ForegroundColor'',''white'',''FontWeight'',''bold'',''String'',a(F(J,2)),''Visible'',''on'')'])
                else
                    eval(['set(handles.Fatherlabel' int2str(J) '_1,''BackgroundColor'',''yellow'',''TooltipString'',''Yellow'',''ForegroundColor'',''black'',''FontWeight'',''bold'',''String'',a(F(J,1)),''Visible'',''on'')'])
                    eval(['set(handles.Fatherlabel' int2str(J) '_2,''BackgroundColor'',''yellow'',''TooltipString'',''Yellow'',''ForegroundColor'',''black'',''FontWeight'',''bold'',''String'',a(F(J,2)),''Visible'',''on'')'])
                end
            else
                eval(['set(handles.Commlabel' int2str(J) ',''ForegroundColor'',''green'',''String'',''match'')'])
                if ~Amb(J)
                    if get(handles.Hppopupmenu,'Value')~=2 && get(handles.Hppopupmenu,'Value')~=3
                        eval(['set(handles.Childlabel' int2str(J) '_1,''TooltipString'',''White'',''String'',a(C(J,1)),''Visible'',''on'')'])
                        eval(['set(handles.Childlabel' int2str(J) '_2,''TooltipString'',''DeepSkyBlue'',''String'',a(C(J,2)),''Visible'',''on'')'])
                        eval(['set(handles.Fatherlabel' int2str(J) '_1,''TooltipString'',''DeepSkyBlue'',''String'',a(F(J,1)),''Visible'',''on'')'])
                        eval(['set(handles.Fatherlabel' int2str(J) '_2,''TooltipString'',''White'',''String'',a(F(J,2)),''Visible'',''on'')'])
                    else
                        eval(['set(handles.Childlabel' int2str(J) '_1,''BackgroundColor'',''white'',''TooltipString'',''White'',''String'',a(C(J,1)),''Visible'',''on'')'])
                        eval(['set(handles.Childlabel' int2str(J) '_2,''BackgroundColor'',''green'',''TooltipString'',''Lime'',''ForegroundColor'',''black'',''FontWeight'',''bold'',''String'',a(C(J,2)),''Visible'',''on'')'])
                        eval(['set(handles.Fatherlabel' int2str(J) '_1,''BackgroundColor'',''green'',''TooltipString'',''Lime'',''ForegroundColor'',''black'',''FontWeight'',''bold'',''String'',a(F(J,1)),''Visible'',''on'')'])
                        eval(['set(handles.Fatherlabel' int2str(J) '_2,''TooltipString'',''White'',''String'',a(F(J,2)),''Visible'',''on'')'])
                    end
                else
                    eval(['set(handles.Childlabel' int2str(J) '_1,''Position'',b,''BackgroundColor'',''magenta'',''TooltipString'',''Fuchsia'',''String'',c,''Visible'',''on'')'])
                    eval(['set(handles.Fatherlabel' int2str(J) '_1,''Position'',b,''BackgroundColor'',''magenta'',''TooltipString'',''Fuchsia'',''String'',c,''Visible'',''on'')'])
                end
            end
        end
        eval(['set(handles.PIlabel' int2str(J) ',''String'',PI(J))'])
        if FHp~=2 && FHp~=3
            eval(['set(handles.RMNElabel' int2str(J) ',''String'',1-EP(J))'])
        end
    end
end
%Set as visible the Comment and the PI panels
set(handles.uipanel5,'Visible','on')
if Rb2==1 && FHp~=1
    if FHp==2 || FHp==3
        str='Sibship Index';
    else
        str='Avuncular Index';
    end
    set(handles.uipanel6,'Title',str)
end
set(handles.uipanel6,'Visible','on')
%Compute the cumulative paternity index (CPI) 
CPI=prod(PI(PI~=0)); 
set(handles.PIlabel19,'String',CPI)
set(handles.PIlabel20,'String','See report')
if CPI>1 %In this case Hp is possible
    if Rb2==1
        if FHp==1
            set(handles.text129,'ForegroundColor','green','String','PATERNITY IS POSSIBLE','Visible','on')
        elseif FHp==2 || FHp==3
            set(handles.text129,'ForegroundColor','green','String','SIBSHIP IS POSSIBLE','Visible','on')
        else
            set(handles.text129,'ForegroundColor','green','String','PATERNAL KINSHIP IS POSSIBLE','Visible','on')
        end
    else
        set(handles.text129,'ForegroundColor','green','String','PARENTAGE IS POSSIBLE','Visible','on')
    end
elseif CPI==1 %in this case Hp=Hd
    if Rb2==1
        if FHp==1
            set(handles.text129,'String','PATERNITY IS INDIFFERENT','Visible','on')
        elseif FHp==2 || FHp==3
            set(handles.text129,'String','SIBSHIP IS INDIFFERENT','Visible','on')
        else
            set(handles.text129,'String','PATERNAL KINSHIP IS INDIFFERENT','Visible','on')
        end
    else
        set(handles.text129,'String','PARENTAGE IS INDIFFERENT','Visible','on')
    end
else %In this case Hd is possible
    if Rb2==1
        if FHp==1
            set(handles.text129,'ForegroundColor','red','String','PATERNITY IS EXCLUDED','Visible','on')
        elseif FHp==2 || FHp==3
            set(handles.text129,'ForegroundColor','red','String','SIBSHIP IS EXCLUDED','Visible','on')
        else
            set(handles.text129,'ForegroundColor','red','String','PATERNAL KINSHIP IS EXCLUDED','Visible','on')
        end
    else
        set(handles.text129,'ForegroundColor','red','String','PARENTAGE IS EXCLUDED','Visible','on')
    end
end
if FHp~=2 && FHp~=3 %compute the Cumulative RMNE and Matching probability and show them
    set(handles.uipanel7,'Visible','on')
    set(handles.RMNElabel19,'String',RMNE)
    set(handles.RMNElabel20,'String',['1:' num2str(round(1/RMNE))])
    set(handles.text127,'Visible','on')
    if Rb2==1
        CMP=prod(MP(MP~=0));
        set(handles.text128,'String',['1:' num2str(round(1/CMP))],'Visible','on')
    end
end



% --- Executes on button press in pushbutton3. (RESET when there is not an incestuous relationship).
function pushbutton3_Callback(~, ~, handles) %#ok<DEFNU>
%Reset all the GUI after a PI calculation.
global Tutflag
Tutflag=0;
clear loci
set(handles.uipanel1,'Visible','on')
set(handles.pushbutton1,'Visible','on')
set(handles.pushbutton3,'Visible','off')
set(handles.Reportmenu1,'Visible','off')
resetgui(handles)
Directparentage_Callback([],[],handles)

% --- Executes on button press in pushbutton4. (COMPUTE! when there is an incestuous relationship).
function pushbutton4_Callback(~, ~, handles) %#ok<DEFNU>
global M C matM Amb ED WE FHp 
global CODISRootFolder
pathname=strcat(CODISRootFolder,'subroutines',filesep);
cd(pathname)
set(handles.pushbutton4,'Visible','off')
%... and make visible the "reset" button
set(handles.pushbutton5,'Visible','on')
set(handles.loadmenu,'Enable','off')
set(handles.Savemenu,'Enable','on')
set(handles.Demomenu,'Enable','off')
set(handles.Optionsmenu,'Enable','off')
set(handles.STRinfomenu,'Enable','off')
set(handles.Reportmenu2,'Visible','on')
%for compatibility and to make easier the implementation of the routine, set as
%0 all the cells that were unselected (in this case the label of the poupmenu
%was 1, remember?)
M(M==1)=0; C(C==1)=0;
ar=[1/4 1/4 1/8 1/8];
CODISINCEST(handles,ar(FHp)) %call the INCEST routine
cd(CODISRootFolder)

%set the gray background
set(handles.uipanel1,'BackgroundColor',[0.702 0.702 0.702])
set(handles.uipanel2,'BackgroundColor',[0.702 0.702 0.702])

for J=1:18 %for each possible STR...
    %switch off the popupmenus
    eval(['set(handles.Motherpopupmenu' int2str(J) '_1,''Visible'',''off'')'])
    eval(['set(handles.Motherpopupmenu' int2str(J) '_2,''Visible'',''off'')'])
    eval(['set(handles.Childpopupmenu' int2str(J) '_1,''Visible'',''off'')'])
    eval(['set(handles.Childpopupmenu' int2str(J) '_2,''Visible'',''off'')'])
    %If there are not informations about child and mother
    if ~all(M(J,:)) || ~all(C(J,:))
        %switch off all the accessory labels
        eval(['set(handles.text' int2str(J) ',''Visible'',''off'')'])
        eval(['set(handles.PIlabel' int2str(J) ',''Visible'',''off'')'])
        eval(['set(handles.RMNElabel' int2str(J) ',''Visible'',''off'')'])
    else %There are informations
        %Set the a,b,c variables to use if Amb(J)=1
        %a=possible alleles for the J-th STR
        %b=position of label1: if Amb(J)=1 move it to the centre of the panel
        %c=concatenation of the observed alleles
        eval(['a=get(handles.Motherpopupmenu' int2str(J) '_1,''String'');']);
        eval(['b=get(handles.Childlabel' int2str(J) '_1,''Position'');']); b(1)=0.3; b(3)=0.4;
        c=strcat(a(C(J,1)),'|',a(C(J,2))); %#ok<NODEF>
        if ~matM(J)
            %If there isn't a match berween mother and child (a very rare case), set the
            %mothers labels with a red background (to highlight the mismatch)
            eval(['set(handles.Motherlabel' int2str(J) '_1,''BackgroundColor'',''red'',''TooltipString'',''Red'',''ForegroundColor'',''white'',''FontWeight'',''bold'',''String'',a(M(J,1)),''Visible'',''on'')'])
            eval(['set(handles.Motherlabel' int2str(J) '_2,''BackgroundColor'',''red'',''TooltipString'',''red'',''ForegroundColor'',''white'',''FontWeight'',''bold'',''String'',a(M(J,1)),''Visible'',''on'')'])
            eval(['set(handles.Childlabel' int2str(J) '_1,''BackgroundColor'',''yellow'',''TooltipString'',''Yellow'',''FontWeight'',''bold'',''String'',a(C(J,1)),''Visible'',''on'')'])
            eval(['set(handles.Childlabel' int2str(J) '_2,''BackgroundColor'',''yellow'',''TooltipString'',''Yellow'',''FontWeight'',''bold'',''String'',a(C(J,2)),''Visible'',''on'')'])
            %There is a match between the mother and the child. In this case,
            %set the child's label background, fontweight and position only if Amb(J)=1
        else
            if ~Amb(J)
                eval(['set(handles.Motherlabel' int2str(J) '_1,''TooltipString'',''White'',''String'',a(M(J,1)),''Visible'',''on'')'])
                eval(['set(handles.Motherlabel' int2str(J) '_2,''TooltipString'',''Lightpink'',''String'',a(M(J,2)),''Visible'',''on'')'])
                eval(['set(handles.Childlabel' int2str(J) '_1,''TooltipString'',''Lightpink'',''String'',a(C(J,1)),''Visible'',''on'')'])
                eval(['set(handles.Childlabel' int2str(J) '_2,''TooltipString'',''DeepSkyBlue'',''String'',a(C(J,2)),''Visible'',''on'')'])
            else
                eval(['set(handles.Motherlabel' int2str(J) '_1,''Position'',b,''BackgroundColor'',''magenta'',''TooltipString'',''Fuchsia'',''String'',c,''Visible'',''on'')'])
                eval(['set(handles.Childlabel' int2str(J) '_1,''Position'',b,''BackgroundColor'',''magenta'',''TooltipString'',''Fuchsia'',''String'',c,''Visible'',''on'')'])
            end
        end
        eval(['set(handles.PIlabel' int2str(J) ',''String'',ED(J))'])
        eval(['set(handles.RMNElabel' int2str(J) ',''String'',WE(J))'])
    end
end
%Set as visible the PI and RMNE panels
a=ED(19); b=prod(WE);
c=(1./(1+1./[a b mean([a b])])).*100;
set(handles.PIlabel19,'String',a)
if ED(19)>1
    set(handles.PIlabel20,'String',sprintf('%0.12f%%',c(1)))
else
    set(handles.PIlabel20,'String',['1:' num2str(round(1/a))])
end
set(handles.RMNElabel19,'String',b)
if b>1
    set(handles.RMNElabel20,'String',sprintf('%0.12f%%',c(2)))
else
    set(handles.RMNElabel20,'String',['1:' num2str(round(1/b))])
end
for I=1:3
    if c(I)>=99.73
        string=',''ForegroundColor'',''red'',''String'',''INCEST IS PROVEN'',''Visible'',''on'')';
    elseif c(I)<99.73 && c(I)>=99.00
        string=',''ForegroundColor'',''red'',''String'',''INCEST IS HIGHLY PROBABLE'',''Visible'',''on'')';
    elseif c(I)<99.00 && c(I)>=95.00
        string=',''ForegroundColor'',''red'',''String'',''INCEST IS VERY PROBABLE'',''Visible'',''on'')';
    elseif c(I)<95.00 && c(I)>=90.00
        string=',''ForegroundColor'',''red'',''String'',''INCEST IS PROBABLE'',''Visible'',''on'')';
    elseif c(I)<90.00 && c(I)>=80.00
        string=',''ForegroundColor'',''red'',''String'',''POSITIVE INDICATION FOR INCEST'',''Visible'',''on'')';
    elseif c(I)<80.00 && c(I)>50.00
        string=',''ForegroundColor'',''red'',''String'',''MORE STRs ARE NEEDED'',''Visible'',''on'')';
    elseif c(I)==50.00
        string=',''ForegroundColor'',''black'',''String'',''INCEST IS INDIFFERENT'',''Visible'',''on'')';
    else
        string=',''ForegroundColor'',''green'',''String'',''INCEST IS EXCLUDED'',''Visible'',''on'')';
    end
    switch I
        case 1
             streval=strcat('set(handles.textED',string);            
        case 2
             streval=strcat('set(handles.textWE',string);            
        case 3
            streval=strcat('set(handles.text129',string);            
    end
    eval(streval)
end
set(handles.uipanel6,'Visible','on')
set(handles.uipanel7,'Visible','on')

% --- Executes on button press in pushbutton5. (RESET when there is an incestuous relationship).
function pushbutton5_Callback(~, ~, handles) %#ok<DEFNU>
%Reset all the GUI after a INCEST calculation.
set(handles.pushbutton4,'Visible','on')
set(handles.pushbutton5,'Visible','off')
set(handles.Reportmenu2,'Visible','off')
resetgui(handles)

%THIS IS THE FILE MENU
% --------------------------------------------------------------------
function Filemenu_Callback(~, ~, ~) %#ok<DEFNU>

% --------------------------------------------------------------------
function loadmenu_Callback(~, ~, handles)  %#ok<DEFNU>
global CODISRootFolder fname
global M C F Pop Kit Rb1 Rb2 FHp FHd 
pathname=strcat(CODISRootFolder,'data',filesep);
cd(pathname)
filename = uigetfile('*.mat','MAT-files (*.mat)', 'Pick a CODIS data file');
if ischar(filename)
    fname=filename(1:end-4);
    load(filename)
    M=CODISdata.M; C=CODISdata.C; F=CODISdata.F;
    Pop=CODISdata.Pop; Kit=CODISdata.Kit;
    Rb1=CODISdata.Rb1; Rb2=CODISdata.Rb2;
    guiloadmenu(handles)
    FHp=CODISdata.FHp; FHd=CODISdata.FHd;
    set(handles.Hppopupmenu,'Value',FHp)
    set(handles.Hdpopupmenu,'Value',FHd)
end
cd(CODISRootFolder); 

% --------------------------------------------------------------------
function Savemenu_Callback(hObject, ~, handles) %#ok<DEFNU>
global CODISRootFolder fname M C F FHp FHd Kitsorted PI EP MP
set(hObject,'Enable','off') %switch off the savemenu
Pop=get(handles.Populationpopupmenu,'Value');
Kit=Kitsorted(get(handles.Kitpopupmenu,'Value')); 
Rb1=isequal(get(handles.Incestyes,'Checked'),'on');
Rb2=isequal(get(handles.Directparentage,'Checked'),'on');
CODISdata=struct('M',M,'C',C,'F',F,'Pop',Pop,'Kit',Kit,'Rb1',Rb1,'Rb2',Rb2,...
    'FHp',FHp,'FHd',FHd,'PI',PI,'RMNE',1-EP,'MP',MP); %#ok<*NASGU>
pathname=strcat(CODISRootFolder,'data',filesep);
%Set the inputdlg options structure
options.Resize='on';
options.WindowStyle='normal';
options.Interpreter='tex';
defans={fname};
fname=char(inputdlg('Input the file name','Filename',1,defans,options));
if ~isempty(fname)
    filename=strcat(pathname,sprintf('%s.mat',fname));
else
    filename='codisdata.mat';
end
save(filename,'CODISdata');

% --------------------------------------------------------------------
function Merge_Callback(~, ~, handles) %#ok<DEFNU>
global CODISRootFolder fname fid 
pathname=strcat(CODISRootFolder,'data',filesep);
cd(pathname)
filename = uigetfile('*.mat','MAT-files (*.mat)', 'Pick the first CODIS data file');
if ischar(filename)
    fname=filename(1:end-4);
    load(filename)
    PI1=CODISdata.PI; RMNE1=CODISdata.RMNE; 
    MP1=CODISdata.MP; Kit1=CODISdata.Kit;
    Pop=CODISdata.Pop;
    pathname=strcat(CODISRootFolder,'alleles',filesep);
    cd(pathname)
    eval(sprintf('load(''%i.mat'',''CODISpop'')',Pop))
    load('kit.mat','KIT')
    cd(CODISRootFolder);
    TF=sum(strcmp(KIT.Kitname,Kit1).*(1:1:length(KIT.Kitname))');
    Idx=KIT.Kitindex(TF,:); a=CODISpop(Idx,7); STR1=char(a(1:18));
    STR1(PI1==0,:)=[]; RMNE1(PI1==0,:)=[]; MP1(PI1==0,:)=[]; PI1(PI1==0,:)=[];
    clear TF a Pop Idx CODISdata 
else
    return
end

pathname=strcat(CODISRootFolder,'reports',filesep);
cd(pathname)
filename=sprintf('%s.html',fname);
fid1 = fopen(filename,'r');

pathname=strcat(CODISRootFolder,'data',filesep);
cd(pathname)
filename = uigetfile('*.mat','MAT-files (*.mat)', 'Pick the second CODIS data file');
if ischar(filename)
    fname=filename(1:end-4);
    load(filename)
    PI2=CODISdata.PI; RMNE2=CODISdata.RMNE; 
    MP2=CODISdata.MP; Kit2=CODISdata.Kit;
    TF=sum(strcmp(KIT.Kitname,Kit2).*(1:1:length(KIT.Kitname))');
    Idx=KIT.Kitindex(TF,:); a=CODISpop(Idx,7); STR2=char(a(1:18));
    STR2(PI2==0,:)=[]; RMNE2(PI2==0,:)=[]; MP2(PI2==0,:)=[]; PI2(PI2==0,:)=[];
    clear TF a Pop Idx CODISpop KIT
else
    return
end
pathname=strcat(CODISRootFolder,'reports',filesep);
cd(pathname)
filename=sprintf('%s.html',fname);
fid2 = fopen(filename,'r');

STR3=char({STR1;STR2}); PI3=[PI1;PI2]; RMNE3=[RMNE1;RMNE2]; MP3=[MP1;MP2]; 
[~,I] = unique(STR3,'rows','stable');
STR=STR3(I,:); PI=PI3(I); RMNE=RMNE3(I); MP=MP3(I); 
clear STR1 STR2 STR3 PI1 PI2 PI3 RMNE1 RMNE2 RMNE3 MP1 MP2 MP3 I

%this function is recalled to generate an html report when we are not in the
%presence of an incestuous relationship
filename=sprintf('%s-TOT.html',fname);
fid = fopen(filename,'w+');
for I=1:3
    fprintf(fid,fgets(fid1));
    fgets(fid2);
end
fgets(fid1); fgets(fid2);
pop=get(handles.Populationpopupmenu,'String');
fprintf(fid,'<P>Population: '); ReportTableRow(pop(CODISdata.Pop,:)); 
fprintf(fid,'<BR>Used kits: %s, %s</P>\n',Kit1,Kit2);
for I=5:11
    fprintf(fid,fgets(fid1));
    fgets(fid2);
end
ok=1; I=1;
while ~isempty(ok)
    tmp1=fgets(fid1);
    ok=strfind(tmp1,deblank(STR(I,:)));
    if ~isempty(ok)
        fprintf(fid,tmp1);
        fprintf(fid,fgets(fid1));
        I=I+1;
    end
end
ok=[]; 
while isempty(ok)
    tmp2=fgets(fid2);
    ok=strfind(tmp2,'CUMULATIVE');
    if isempty(ok);
        nomatch=1;
        while nomatch==1
            if ~isempty(strfind(tmp2,deblank(STR(I,:))))
                nomatch=0;
                fprintf(fid,tmp2);
                fprintf(fid,fgets(fid2));
                I=I+1;
            end
        end
    end
end
fprintf(fid,tmp1(1:74));
CPI=prod(PI);
if CPI>1
    fprintf(fid,'%0.2f</TH></TR>\n',CPI);
else
    fprintf(fid,'%1.4e</TH></TR>\n',CPI);
end
fgets(fid1); fgets(fid2); fprintf(fid,'</TABLE>\n');

if CPI>1 %In this case Hp is possible
    fontcolor='Green';
    if CODISdata.Rb2==1
        if CODISdata.FHp==1
            str='PATERNITY IS POSSIBLE';
        elseif CODISdata.FHp==2 || CODISdata.FHp==3
            str='SIBSHIP IS POSSIBLE';
        else
            str='PATERNAL KINSHIP IS POSSIBLE';
        end
    else
        str='PARENTAGE IS POSSIBLE';
    end
elseif CODISdata.CPI==1 %in this case Hp=Hd
    fontcolor='Black';
    if CODISdata.Rb2==1
        if CODISdata.FHp==1
            str='PATERNITY IS INDIFFERENT';
        elseif CODISdata.FHp==2 || CODISdata.FHp==3
            str='SIBSHIP IS INDIFFERENT';
        else
            str='PATERNAL KINSHIP IS INDIFFERENT';
        end
    else
        str='PARENTAGE IS INDIFFERENT';
    end
else %In this case Hd is possible
    fontcolor='Red';
    if CODISdata.Rb2==1
        if CODISdata.FHp==1
            str='PATERNITY IS EXCLUDED';
        elseif CODISdata.FHp==2 || CODISdata.FHp==3
            str='SIBSHIP IS EXCLUDED';
        else
            str='PATERNAL KINSHIP IS EXCLUDED';
        end
    else
        str='PARENTAGE IS EXCLUDED';
    end
end
fgets(fid1); fgets(fid2); fprintf(fid,'<P style="font-size:200%%;color:%s;">%s</P>\n',fontcolor,str);
L=[5 10 20 30 40 50 60 70 80 90 95];
PrO=L./fliplr(L);
PsO=(1./(1+1./(PrO.*CPI))).*100;
for I=1:21
    fprintf(fid,fgets(fid1));
    fgets(fid2);
end
for I=1:5
    if PsO(I)>1
        fprintf(fid,'<td style="text-align: center;">%0.10f%%</td>\n',PsO(I));
    else
        fprintf(fid,'<td style="text-align: center;">1:%i</td>\n',1/PsO(I));
    end
    fgets(fid1); fgets(fid2);
end
for I=1:8
    fprintf(fid,fgets(fid1));
    fgets(fid2);
end
if PsO(6)>1
    fprintf(fid,'<td colspan="5" rowspan="1" style="text-align: center; background-color: yellow;">%0.10f%%</td>\n',PsO(6));
else
    fprintf(fid,'<td colspan="5" rowspan="1" style="text-align: center; background-color: yellow;">1:%i</td>\n',1/PsO(6));
end
fgets(fid1); fgets(fid2);
for I=1:20
    fprintf(fid,fgets(fid1));
    fgets(fid2);
end
for I=7:11
    if PsO(I)>1
        fprintf(fid,'<td style="text-align: center;">%0.10f%%</td>\n',PsO(I));
    else
        fprintf(fid,'<td style="text-align: center;">1:%i</td>\n',1/PsO(I));
    end
    fgets(fid1); fgets(fid2);
end
fprintf(fid,fgets(fid1)); fgets(fid2);
if CODISdata.FHp==1
    for I=1:2
        fprintf(fid,fgets(fid1)); fgets(fid2);
    end
    ok=1; I=1;
    while ~isempty(ok)
        tmp1=fgets(fid1);
        ok=strfind(tmp1,deblank(STR(I,:)));
        if ~isempty(ok)
            fprintf(fid,tmp1);
            fprintf(fid,fgets(fid1));
            I=I+1;
        end
    end
    ok=[];
    while isempty(ok)
        tmp2=fgets(fid2);
        ok=strfind(tmp2,'CUMULATIVE');
        if isempty(ok);
            nomatch=1;
            while nomatch==1
                if ~isempty(strfind(tmp2,deblank(STR(I,:))))
                    nomatch=0; fprintf(fid,tmp2); fprintf(fid,fgets(fid2)); I=I+1;
                end
            end
        end
    end
    fprintf(fid,'<TR><TH>CUMULATIVE</TH>');
    fprintf(fid,'<TH COLSPAN=2 style="background-color:Silver;"></TH><TH style="background-color:Darkorange;">%1.4e</TH>',prod(1-RMNE));
    fprintf(fid,'<TH COLSPAN=2 style="background-color:Silver;"></TH><TH style="background-color:Darkorange;">%1.4e</TH></TR>\n',prod(MP));
    fprintf(fid,'<TR><TH>PROBABILITY</TH>');
    fprintf(fid,'<TH COLSPAN=2 style="background-color:Silver;"></TH><TH style="background-color:Darkorange;">1:%0.0f</TH>',1/prod(1-RMNE));
    fprintf(fid,'<TH COLSPAN=2 style="background-color:Silver;"></TH><TH style="background-color:Darkorange;">1:%0.0f</TH></TR>',1/prod(MP));
    fprintf(fid,'</TABLE>\n');
end
fprintf(fid,'</BODY></HTML>\n');
fclose('all');
cd(CODISRootFolder)

% --------------------------------------------------------------------
function quitcodis_Callback(~, ~, ~) %#ok<DEFNU>
delete(gcf)

%THIS IS THE DEMO MENU
% --------------------------------------------------------------------
function Demomenu_Callback(~, ~, ~) %#ok<DEFNU>

function Trioatt_Callback(~, ~, handles) %#ok<DEFNU>
%Load the dataset to show a paternity attribuition with a trio (mother, child
%and alleged father)
global M C F Pop Kit Rb1 Rb2 FHp FHd 
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata1.mat','CODISdata')
cd(CODISRootFolder); 
M=CODISdata.M; C=CODISdata.C; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit;
Rb1=CODISdata.Rb1; Rb2=CODISdata.Rb2;
democommon(0,handles)
FHp=CODISdata.FHp; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

function Duoatt_Callback(~, ~, handles) %#ok<DEFNU>
%Load the dataset to show a paternity attribuition with a duo (child
%and alleged father)
global M C F Pop Kit Rb1 Rb2 FHp FHd
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata1.mat','CODISdata')
cd(CODISRootFolder); 
M=ones(size(C)); C=CODISdata.C; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit;
Rb1=CODISdata.Rb1; Rb2=CODISdata.Rb2;
democommon(0,handles)
FHp=CODISdata.FHp; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

function Trioexc_Callback(~, ~, handles) %#ok<DEFNU>
%Load the dataset to show a paternity exclusion with a trio (mother, child
%and alleged father)
global M C F Pop Kit Rb1 Rb2 FHp FHd 
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata2.mat','CODISdata')
cd(CODISRootFolder); 
M=CODISdata.M; C=CODISdata.C; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit;
Rb1=CODISdata.Rb1; Rb2=CODISdata.Rb2;
democommon(0,handles)
FHp=CODISdata.FHp; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

function Duoexc_Callback(~, ~, handles) %#ok<DEFNU>
%Load the dataset to show a paternity exclusion with a duo (mother, child
%and alleged father)
global M C F Pop Kit Rb1 Rb2 FHp FHd 
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata2.mat','CODISdata')
cd(CODISRootFolder); 
M=ones(size(C)); C=CODISdata.C; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit;
Rb1=CODISdata.Rb1; Rb2=CODISdata.Rb2;
democommon(0,handles)
FHp=CODISdata.FHp; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

% --------------------------------------------------------------------
function TrioAttRP_Callback(~, ~, handles)%#ok<DEFNU>
%Load the dataset to show a paternity attribuition with a trio (mother, child
%and alleged father)
global M C F Pop Kit Rb1 Rb2 FHp FHd 
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata8.mat','CODISdata')
cd(CODISRootFolder); 
M=CODISdata.M; C=CODISdata.C; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit;
Rb1=CODISdata.Rb1; Rb2=CODISdata.Rb2;
democommon(0,handles)
FHp=CODISdata.FHp; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

% --------------------------------------------------------------------
function Fullsibs_Callback(~, ~, handles) %#ok<DEFNU>
%Load the dataset to show a Sibship attribuition
global M C F Pop Kit Rb1 Rb2 FHp FHd 
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata3.mat','CODISdata')
cd(CODISRootFolder); 
M=CODISdata.M; C=CODISdata.C; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit;
Rb1=CODISdata.Rb1; Rb2=CODISdata.Rb2;
democommon(1,handles)
FHp=CODISdata.FHp; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

% --------------------------------------------------------------------
function MCswapped_Callback(~, ~, handles) %#ok<DEFNU>
%Load the dataset to show a case were mother and child samples were swapped
global M C F Pop Kit Rb1 Rb2 FHp FHd 
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);cd(pathname)
load('democodisdata1.mat','CODISdata')
cd(CODISRootFolder); 
M=CODISdata.M; C=CODISdata.C; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit;
Rb1=CODISdata.Rb1; Rb2=CODISdata.Rb2;
tmp=M; M=C; C=tmp; clear tmp
democommon(0,handles)
FHp=CODISdata.FHp; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

% --------------------------------------------------------------------
function Nomother_Callback(~, ~, handles) %#ok<DEFNU>
%Load the dataset to show the check if "mater certa est"
global M C F Pop Kit Rb1 Rb2 FHp FHd 
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata7.mat','CODISdata')
cd(CODISRootFolder); 
M=CODISdata.M; C=CODISdata.C; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit;
Rb1=CODISdata.Rb1; Rb2=CODISdata.Rb2;
democommon(0,handles)
FHp=CODISdata.FHp; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

% --------------------------------------------------------------------
function Incestdemo_Callback(~, ~, ~) %#ok<DEFNU>
% --------------------------------------------------------------------
function BSincestdemo_Callback(~, ~, ~)%#ok<DEFNU>
% --------------------------------------------------------------------
function FBcheckdemo_Callback(~, ~, handles) %#ok<DEFNU>
%Load the dataset to show a Sibship attribuition
global M C F Pop Kit Rb1 Rb2 FHp FHd 
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata9.mat','CODISdata')
cd(CODISRootFolder); 
M=zeros(size(CODISdata.M)); C=CODISdata.M; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit;
Rb1=0; Rb2=1;
democommon(1,handles)
FHp=2; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)
% --------------------------------------------------------------------
function STRincest_Callback(~, ~, handles) %#ok<DEFNU>
global M C F Pop Kit Rb1 Rb2 FHp FHd 
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata9.mat','CODISdata')
cd(CODISRootFolder); 
M=CODISdata.M; C=CODISdata.C; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit;
Rb1=CODISdata.Rb1; Rb2=CODISdata.Rb2;
democommon(0,handles)
FHp=CODISdata.FHp; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

% --------------------------------------------------------------------
function WEincest_Callback(~, ~, handles) %#ok<DEFNU>
global M C F Pop Kit Rb1 Rb2 FHp FHd 
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata9.mat','CODISdata')
cd(CODISRootFolder); 
M=CODISdata.M; C=CODISdata.C; F=zeros(size(CODISdata.F));
Pop=CODISdata.Pop; Kit=CODISdata.Kit;
Rb1=1; Rb2=CODISdata.Rb2;
democommon(0,handles)
FHp=2; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

% --------------------------------------------------------------------
function Tutdemo_Callback(~, ~, ~) %#ok<DEFNU>
global Tutflag
Tutflag=1;

%Tutankhamun's parents were full sibs (TUTDEMO4)
function Tut1_Callback(~, ~, handles) %#ok<DEFNU>
global M C F Pop Kit Rb1 Rb2 FHp FHd 
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata4.mat','CODISdata')
cd(CODISRootFolder); 
M=ones(size(C)); C=CODISdata.M; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit; 
Rb1=CODISdata.Rb1; Rb2=CODISdata.Rb2;
set(handles.uipanel2,'Title','The Younger Lady mummy (KV35)')
set(handles.uipanel3,'Title','Akhenaten')
democommon(1,handles)
FHp=CODISdata.FHp; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

%Tutankhamun was an incest product (TUTDEMO 5)
function Tut2_Callback(~, ~, handles) %#ok<DEFNU>
%Load the dataset to show a Sibship attribuition
global M C F Pop Kit Rb1 Rb2 FHp FHd 
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata4.mat','CODISdata')
cd(CODISRootFolder); 
M=CODISdata.M; C=CODISdata.C; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit; 
Rb1=1; Rb2=1;
set(handles.uipanel1,'Title','The Younger Lady mummy (KV35)')
set(handles.uipanel2,'Title','Tutankhamun')
democommon(0,handles)
FHp=2; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

%Tutankhamun's grandmother family (TUTDEMO 1)
function Tut3_Callback(~, ~, handles) %#ok<DEFNU>
global M C F Pop Kit Rb1 Rb2 FHp FHd 
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata5.mat','CODISdata')
cd(CODISRootFolder); 
M=CODISdata.M; C=CODISdata.C; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit;
Rb1=CODISdata.Rb1; Rb2=CODISdata.Rb2;
set(handles.uipanel1,'Title','Thuya')
set(handles.uipanel2,'Title','The Elder Lady mummy (Tiye)')
set(handles.uipanel3,'Title','Yuya')
democommon(0,handles)
FHp=CODISdata.FHp; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

%Akhenaten's (Tut's father) family (TUTDEMO 2)
function Tut4_Callback(~, ~, handles) %#ok<DEFNU>
global M C F Pop Kit Rb1 Rb2 FHp FHd
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata6.mat','CODISdata')
cd(CODISRootFolder); 
M=CODISdata.M; C=CODISdata.C; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit;
Rb1=CODISdata.Rb1; Rb2=CODISdata.Rb2;
set(handles.uipanel1,'Title','The Elder Lady mummy (Tiye)')
set(handles.uipanel2,'Title','Akhenaten')
set(handles.uipanel3,'Title','Amenhotep III')
democommon(0,handles)
FHp=CODISdata.FHp; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

%Tut's mother family (TUTDEMO 3)
function Tut5_Callback(~, ~, handles) %#ok<DEFNU>
global M C F Pop Kit Rb1 Rb2 FHp FHd 
global CODISRootFolder 
pathname=strcat(CODISRootFolder,'demos',filesep);
cd(pathname)
load('democodisdata4.mat','CODISdata')
KV35=CODISdata.M;
load('democodisdata6.mat','CODISdata')
cd(CODISRootFolder); 
M=CODISdata.M; C=KV35; F=CODISdata.F;
Pop=CODISdata.Pop; Kit=CODISdata.Kit;
Rb1=CODISdata.Rb1; Rb2=CODISdata.Rb2;
set(handles.uipanel1,'Title','The Elder Lady mummy (Tiye)')
set(handles.uipanel2,'Title','The Younger Lady mummy (KV35)')
set(handles.uipanel3,'Title','Amenhotep III')
democommon(0,handles)
FHp=CODISdata.FHp; FHd=CODISdata.FHd;
set(handles.Hppopupmenu,'Value',FHp)
set(handles.Hdpopupmenu,'Value',FHd)

function democommon(flag,handles) %this is the actions runned by all demos
guiloadmenu(handles)
set(handles.Filemenu,'Enable','off')
set(handles.Demomenu,'Enable','off')
set(handles.STRinfomenu,'Enable','off')
if flag==1
    set(handles.uipanel1,'Visible','off')
end

% --------------------------------------------------------------------
function Optionsmenu_Callback(~, ~, ~) %#ok<DEFNU>

% --------------------------------------------------------------------
function Parentagemenu_Callback(~, ~, ~)%#ok<DEFNU>

% --------------------------------------------------------------------
function Directparentage_Callback(~, ~, handles)
global FHp FHd Hpstring Hdstring Parentage
Parentage=1; FHp=1; FHd=1;
set(handles.Directparentage,'Checked','on')
set(handles.Reverseparentage,'Checked','off')
set(handles.uipanel8,'Title','Hp   Proband is paternally related to child as')
set(handles.uipanel9,'Title','Hd   Proband is paternally related to child as')
set(handles.Hppopupmenu,'String',Hpstring,'Value',FHp)
set(handles.Hdpopupmenu,'String',Hdstring,'Value',FHd)
set(handles.uipanel6,'Title','Paternity Index','Position',[0.719 0.064 0.13 0.823])
set(handles.uipanel7,'Title','Random Man not Excluded','Position',[0.855 0.064 0.13 0.823])

% --------------------------------------------------------------------
function Reverseparentage_Callback(~, ~, handles)
global FHp FHd Parentage
FHp=1; FHd=1; Parentage=0;
set(handles.Directparentage,'Checked','off')
set(handles.Reverseparentage,'Checked','on')
set(handles.uipanel8,'Title','Hp   Couple is related to child as')
set(handles.uipanel9,'Title','Hd   Couple is related to child as')
set(handles.Hppopupmenu,'String','Parents-Child','Value',FHp)
set(handles.Hdpopupmenu,'String','Unrelated','Value',FHd)
set(handles.uipanel6,'Title','Parentage Index','Position',[0.719 0.064 0.13 0.823])
set(handles.uipanel7,'Title','Random Couples not Excluded','Position',[0.855 0.064 0.13 0.823])

% --------------------------------------------------------------------
function Incestmenu_Callback(~, ~, ~) %#ok<DEFNU>

% --------------------------------------------------------------------
function Incestyes_Callback(~, ~, handles)
global FHp 
FHp=1;
set(handles.Incestyes,'Checked','on')
set(handles.Incestno,'Checked','off')
set(handles.uipanel3,'Visible','off')
set(handles.uipanel9,'Visible','off')
set(handles.uipanel8,'Title','Type of incestuous relationship proposed')
set(handles.Hppopupmenu,'String',{'Father-Child','Brother-Sister','Half Sibs','Uncle/Aunt-Niece/Nephew'},'Value',FHp)
set(handles.uipanel6,'Title','Incest Index by Edwards','Position',[0.468 0.064 0.13 0.823])
set(handles.uipanel7,'Title','Incest Index by Wenk','Position',[0.604 0.064 0.13 0.823])
set(handles.pushbutton1,'Visible','off')
set(handles.pushbutton4,'Visible','on')
set(handles.Parentagemenu,'Enable','off')

% --------------------------------------------------------------------
function Incestno_Callback(~, ~, handles)
set(handles.Incestyes,'Checked','off')
set(handles.Incestno,'Checked','on')
set(handles.uipanel3,'Visible','on')
set(handles.uipanel9,'Visible','on')
set(handles.Parentagemenu,'Enable','on')
Directparentage_Callback([], [], handles)
set(handles.pushbutton4,'Visible','off')
set(handles.pushbutton1,'Visible','on')

%THIS IS THE STRINFO MENU
% --------------------------------------------------------------------
function STRinfomenu_Callback(~, ~, ~) %#ok<DEFNU>

% ALLELES FREQUENCIES
function FreqAllmenu_Callback(~, ~, ~) %#ok<DEFNU>

function All_Callback(~, ~, handles) %#ok<DEFNU>
for I=1:23
    Reportallele(handles,I)
end

function D8S1179_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,1)

function D21S11_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,2)

function D7S820_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,3)

function CSF1PO_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,4)

function D3S1358_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,5)

function TH01_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,6)

function D13S317_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,7)

function D16S539_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,8)

function vWA_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,9)

function TPOX_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,10)

function D18S51_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,11)

function D5S818_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,12)

function FGA_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,13)

function D2S1338_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,14)

function D19S433_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,15)

function PENTA_D_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,16)

function PENTA_E_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,17)

function SE33_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,18)

function D1S1656_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,19)

function D2S441_Callback(~, ~, handles) %#ok<DEFNU>
Idx=KIT.Kitindex(TF,:); J=1;
Reportallele(handles,20)

function D10S1248_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,21)

function D12S391_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,22)

function D22S1045_Callback(~, ~, handles) %#ok<DEFNU>
Reportallele(handles,23)

function Reportallele(handles,S) 
%this function is recalled to generate an html report for STRinfomenu
global fid CODISpop CODISRootFolder
pathname=strcat(CODISRootFolder,'stats',filesep);
%Set the inputdlg options structure
cd(pathname)
STRstr1=strcat('Alleles frequencies for the STR marker: ',CODISpop{S,7});
title=strcat(STRstr1,'_report.html');
fid = fopen(title,'w+');
fprintf(fid,'<HTML><HEAD><TITLE>COmbined DNA Index System Report</TITLE></HEAD>\n');
fprintf(fid,'<BODY style="font-face:Arial;font-size=14px">\n');
fprintf(fid,'<H1 style="text-align:center;font-weight:bold;">CODIS Report</H1>\n');
fprintf(fid,'<H2 style="text-align:center;font-weight:bold;">%s</H2>\n',STRstr1);
x=get(handles.Populationpopupmenu,'Value');
pop=get(handles.Populationpopupmenu,'String');
fprintf(fid,'<P>Population: '); ReportTableRow(pop(x,:)); fprintf(fid,'</P><BR>\n');
f=CODISpop{S,1};
if isempty(f) 
    fprintf(fid,'For the selected population, there aren''t data for this STR\n');
else
    fprintf(fid,'<P>Counted Alleles: %i</P>\n',CODISpop{S,2});
    fprintf(fid,'<P>Paternal Mutation Rate: %0.4e</P>\n',CODISpop{S,3});
    fprintf(fid,'<P>Maternal Mutation Rate: %0.4e</P><BR>\n',CODISpop{S,4});
    fprintf(fid,'<TABLE BORDER="2" CELLPADDING="4" CELLSPACING="0" style="text-align:center;">\n');
    fprintf(fid,'<TR><TH><P style="color:red;">Allele</P></TH>');
    fprintf(fid,'<TH><P style="color:red;">Frequency</P></TH></TR>');
    A=CODISpop{S,6};
    for J=2:length(A),get(handles.Kitpopupmenu,'String')
        fprintf(fid,'<TR><TD><P>'); ReportTableRow(A(J)); fprintf(fid,'</P></TD>');
        fprintf(fid,'<TD><P>%0.4f</P></TD></TR>\n',f(J-1));
    end
    fprintf(fid,'<TR><TH><P style="color:red;">Sum</P></TH>');
    fprintf(fid,'<TH><P style="color:red;">%0.4f</P></TH></TR>\n',sum(f));
    fprintf(fid,'</TABLE>\n');
end
fprintf(fid,'</BODY></HTML>\n');
fclose(fid);
cd(CODISRootFolder)


%FORENSIC STATISTICS
function Forstats_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr1
STRstr1='Forensic Statistics';

%MATCHING PROBABILITY (GENERAL) AND POWER OF DISCRIMINATION
function pMMenu_Callback(~, ~, ~) %#ok<DEFNU>

function pMMenuS_Callback(~, ~, handles) %#ok<DEFNU>
%This function compute the probability of match (PM) for the selected kit.
%For each STR, the PM is the fraction of the population that share the same
%genotype.
%Of course, the complementary is the Power of Discrimination (PD).
global A STRstr2
STRinfoguiprep(handles)
%Modify the Paternity index and the RMNE Panels to show the required info
set(handles.uipanel6,'Title','Probability of Match','Visible','on')
set(handles.uipanel7,'Title','Power of discrimination','Visible','on')
set(handles.PIlabel19,'String','Overall pM')
set(handles.RMNElabel19,'String','Overall PD')
STRstr2='Matching Probability and Power of Discrimination';
parameters(2,handles,1)

%show the results
OA=prod(A(~isnan(A))); 
set(handles.PIlabel20,'String',sprintf('1:%0.5e',1/OA))
set(handles.RMNElabel20,'String',sprintf('%0.10f%%',100*(1-OA)))

function pMMenuWB_Callback(~, ~, handles) %#ok<DEFNU>
%This function compute the worst and the best 
%probability of match (PM) for the selected kit.
%For each STR, the PM is the fraction of the population that share the same
%genotype.
global A B STRstr2
STRinfoguiprep(handles)
%Modify the Paternity index and the RMNE Panels to show the required info
set(handles.uipanel6,'Title','Worst Probability of Match','Visible','on')
set(handles.uipanel7,'Title','Best Probability of Match','Visible','on')
set(handles.PIlabel19,'String','Overall')
set(handles.RMNElabel19,'String','Overall')
STRstr2='The Worst and the Best Matching Probability';
parameters(2,handles,2)
%show the results
set(handles.PIlabel20,'String',sprintf('1:%0.10e',1/prod(A(~isnan(A)))))
set(handles.RMNElabel20,'String',sprintf('1:%0.10e',1/prod(B(~isnan(B)))))

% POLYMORPHISM INFORMATION CONTENT
function PIC_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2 A
STRinfoguiprep(handles)
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PIC','Visible','on')
set(handles.PIlabel19,'String','Overall PIC')
STRstr2='Polymorphism Information Content (PIC)';
parameters(1,handles,3)
%show the results
set(handles.PIlabel20,'String',sprintf('%0.12f%%',100*(1-prod(A(~isnan(A))))))

% --------------------------------------------------------------------
%PATERNITY STATISTICS
function Patstats_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr1
STRstr1='Paternity Statistics';

%TYPICAL PATERNITY INDEX
function TPImenu_Callback(~, ~, handles) %#ok<DEFNU>
%This function shows a Typical Paternity index that is possible to have for
%the selected kit, in case of paternity attribuition.
global A STRstr2
STRinfoguiprep(handles)
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','Typical Paternity Index','Visible','on')
STRstr2='Typical Paternity Index (TPI)';
parameters(1,handles,4)
%show the results
CTPI=prod(A(~isnan(A))); PCTPI=(1/(1+1/CTPI));
set(handles.PIlabel19,'String',CTPI)
set(handles.PIlabel20,'String',sprintf('%0.12f%%',100*PCTPI))

%MEAN EXCLUSION CHANCE
function MECmenu_Callback(~, ~, handles) %#ok<DEFNU>
%This function shows a Mean exclusion chance that is possible to have for
%the selected kit, in case of paternity exclusion.
global A STRstr2
STRinfoguiprep(handles)
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','Mean Exclusion Chance','Visible','on')
STRstr2='Mean Eclusion Chance (MEC)';
parameters(1,handles,5)
%show the results
set(handles.PIlabel19,'String','Overall MEC')
set(handles.PIlabel20,'String',sprintf('%0.12f%%',100*(1-prod(1-A(~isnan(A))))))

%POWER OF EXCLUSION...
function PEmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2
STRstr2='Power of exclusion-';

%...OF A RANDOM MAN---
function RMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 phi
phi=1;
STRstr2=strcat(STRstr2,'Random man-');

%--- WITH MOTHER
function PEWMmenu_Callback(~, ~, ~) %#ok<DEFNU>
%This function compute the power of exclusion, for the selected kit, when the
%mother is present. For each STR, compute the Proportion of Not excluded (PNE).
%This proportion shows how many random man could be considered as true father
%because their genotype is compatible with the allele not inherited from the
%mother. Of course, the complementary is the Power of the exclusion (PE).
%This value is a general PE, without considering a specific trio.
%Modify the Paternity index Panel to show the required info
global STRstr2 motherflag
motherflag=1;
STRstr2=strcat(STRstr2,'Mother is available-');

% --------------------------------------------------------------------
function RMPwrExclNomutmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2 
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
set(handles.uipanel6,'Title','PERM with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function RMPwrExcl2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2 
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
set(handles.uipanel6,'Title','PERM with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function RMPwrExcl3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2 
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
set(handles.uipanel6,'Title','PERM with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);


%--- WITHOUT MOTHER
function PENMmenu_Callback(~, ~, ~) %#ok<DEFNU>
%This function compute the power of exclusion, for the selected kit, when the
%mother is absent. For each STR, compute the Proportion of Not excluded (PNE).
%This proportion shows how many random man could be considered as true father
%because their genotype is compatible with the observed child's genotype.
%Of course, the complementary is the Power of the exclusion (PE).
%This value is a general PE, without considering a specific duo.
%Modify the Paternity index Panel to show the required info
global STRstr2 motherflag
motherflag=-1;
STRstr2=strcat(STRstr2,'Mother is not available-');

% --------------------------------------------------------------------
function RMPwrExclNomutNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2 
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
set(handles.uipanel6,'Title','PERM without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function RMPwrExcl2lociNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
set(handles.uipanel6,'Title','PERM without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function RMPwrExcl3lociNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2 
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
set(handles.uipanel6,'Title','PERM without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);


% ...OF A RELATIVE OF THE TRUE FATHER (TF) ---
function Relmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 
STRstr2=strcat(STRstr2,'A realative of the true father-');

%MOTHER UNRELATED
function MUNmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Mother unrelated-');

%UPPER IN THE GENEALOGIC TREE
function Ascmenu_Callback(~, ~, ~) %#ok<DEFNU>

% ---GRANFATHER (TF'S FATHER)^^^
function GFmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 phi
phi=1/2;
STRstr2=strcat(STRstr2,'Grandfather (True father''s father)-');

% ^^^WITH MOTHER
function GFWMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=1;
STRstr2=strcat(STRstr2,'Mother is available-');

% --------------------------------------------------------------------
function GFWMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2 
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Granpa - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function GFWM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2 
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Granpa - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function GFWM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2 
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Granpa - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);

% ^^^WITHOUT MOTHER
function GFNMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=-1;
STRstr2=strcat(STRstr2,'Mother is not available-');

% --------------------------------------------------------------------
function GFNMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2 
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Granpa - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function GFNM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Granpa - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function GFNM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2 
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Granpa - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);


% ---GREAT-UNCLE (Grandfather's full brother)^^^
function GUFBmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 phi
phi=3/4;
STRstr2=strcat(STRstr2,'Great-Uncle (Grandfather''s full sib)-');

% ^^^WITH MOTHER
function GUFBWMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=1;
STRstr2=strcat(STRstr2,'Mother is available-');

% --------------------------------------------------------------------
function GUFBWMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2 
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Great-Uncle (FB) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function GUFBWM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2 
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Great-Uncle (FB) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function GUFBWM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2 
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Great-Uncle (FB) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);

% ^^^WITHOUT MOTHER
function GUFBNMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=-1;
STRstr2=strcat(STRstr2,'Mother is not available-');

% --------------------------------------------------------------------
function GUFBNMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Great-Uncle (FB) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function GUFBNM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Great-Uncle (FB) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function GUFBNM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Great-Uncle (FB) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);

% ---GREAT-UNCLE (Grandfather's half brother)^^^
function GUHBmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 phi
phi=7/8;
STRstr2=strcat(STRstr2,'Great-Uncle (Grandfather''s half sib)-');

% ^^^WITH MOTHER
function GUHBWMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=1;
STRstr2=strcat(STRstr2,'Mother is available-');

% --------------------------------------------------------------------
function GUHBWMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Great-Uncle (HB) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function GUHBWM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Great-Uncle (HB) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function GUHBWM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Great-Uncle (HB) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);

% ^^^WITHOUT MOTHER
function GUHBNMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=-1;
STRstr2=strcat(STRstr2,'Mother is not available-');

% --------------------------------------------------------------------
function GUHBNMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Great-Uncle (HB) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function GUHBNM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Great-Uncle (HB) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function GUHBNM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Great-Uncle (HB) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);

% COLLATERAL IN THE GENEALOGIC TREE
function Collmenu_Callback(~, ~, ~) %#ok<DEFNU>

% ---UNCLE (TF'S FULL BROTHER)^^^
function UFBmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 phi
phi=1/2;
STRstr2=strcat(STRstr2,'Uncle (True father''s full sib)-');

% ^^^WITH MOTHER
function UFBWMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=1;
STRstr2=strcat(STRstr2,'Mother is available-');

% --------------------------------------------------------------------
function UFBWMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Uncle (FB) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function UFBWM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Uncle (FB) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function UFBWM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Uncle (FB) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);

% ^^^WITHOUT MOTHER
function UFBNMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=-1;
STRstr2=strcat(STRstr2,'Mother is not available-');

% --------------------------------------------------------------------
function UFBNMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Uncle (FB) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function UFBNM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Uncle (FB) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function UFBNM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Uncle (FB) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);


% ---UNCLE (TF'S HALF BROTHER)^^^
function UHBmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 phi
phi=3/4;
STRstr2=strcat(STRstr2,'Uncle (True father''s half sib)-');

% ^^^WITH MOTHER
function UHBWMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=1;
STRstr2=strcat(STRstr2,'Mother is available-');

% --------------------------------------------------------------------
function UHBWMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Uncle (HB) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function UHBWM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Uncle (HB) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function UHBWM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Uncle (HB) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);

% ^^^WITHOUT MOTHER
function UHBNMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=-1;
STRstr2=strcat(STRstr2,'Mother is not available-');

% --------------------------------------------------------------------
function UHBNMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Uncle (HB) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function UHBNM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Uncle (HB) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function UHBNM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Uncle (HB) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);


%---TF'S FIRST COUSIN^^^
function FCmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 phi
phi=7/8;
STRstr2=strcat(STRstr2,'True father''s first cousin-');

% ^^^WITH MOTHER
function FCWMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=1;
STRstr2=strcat(STRstr2,'Mother is available-');

% --------------------------------------------------------------------
function FCWMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER TF''s First Cousin - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function FCWM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER TF''s First Cousin - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function FCWM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER TF''s First Cousin - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);

% ^^^WITHOUT MOTHER
function FCNMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=-1;
STRstr2=strcat(STRstr2,'Mother is not available-');

% --------------------------------------------------------------------
function FCNMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER TF''s First Cousin - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function FCNM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER TF''s First Cousin - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function FCNM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER TF''s First Cousin - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);

% LOWER IN THE GENEALOGIC TREE
function Demenu_Callback(~, ~, ~) %#ok<DEFNU>

%---NEPHEW (SON OF TF'S FULL SIB)^^^
function NFSmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 phi
phi=3/4;
STRstr2=strcat(STRstr2,'True father''s nephew (son of a TF''s full sib)-');

% ^^^WITH MOTHER
function NFSWMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=1;
STRstr2=strcat(STRstr2,'Mother is available-');

% --------------------------------------------------------------------
function NFSWMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Nephew (FS) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function NFSWM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Nephew (FS) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function NFSWM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Nephew (FS) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);

% ^^^WITHOUT MOTHER
function NFSNMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=-1;
STRstr2=strcat(STRstr2,'Mother is not available-');

% --------------------------------------------------------------------
function NFSNMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Nephew (FS) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function NFSNM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Nephew (FS) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function NFSNM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Nephew (FS) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);

%---NEPHEW (SON OF TF'S HALF SIB)^^^
function NHSmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 phi
phi=7/8;
STRstr2=strcat(STRstr2,'True father''s nephew (son of a TF''s half sib)-');

% ^^^WITH MOTHER
function NHSWMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=1;
STRstr2=strcat(STRstr2,'Mother is available-');

% --------------------------------------------------------------------
function NHSWMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Nephew (HS) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function NHSWM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Nephew (HS) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function NHSWM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Nephew (HS) - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);

% ^^^WITHOUT MOTHER
function NHSNMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2 motherflag
motherflag=-1;
STRstr2=strcat(STRstr2,'Mother is not available-');

% --------------------------------------------------------------------
function NHSNMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Nephew (HS) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,0);

% --------------------------------------------------------------------
function NHSNM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Nephew (HS) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,2);

% --------------------------------------------------------------------
function NHSNM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
%Modify the Paternity index Panel to show the required info
set(handles.uipanel6,'Title','PER Nephew (HS) - without mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
PwrEx(handles,3);

% MOTHER RELATED
function MRmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Mother related-');

% A CHILD'S ELDER BROTHER
function EBmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'An elder full sib-');

%WITH MOTHER
function EBWMmenu_Callback(~, ~, ~) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Mother is available-');

% --------------------------------------------------------------------
function EBWMNMmenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 1 or more loci');
PwrExEB(handles,0)

% --------------------------------------------------------------------
function EBWM2locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 2 or more loci');
PwrExEB(handles,2)

% --------------------------------------------------------------------
function EBWM3locimenu_Callback(~, ~, handles) %#ok<DEFNU>
global STRstr2
STRstr2=strcat(STRstr2,'Exclusion at 3 or more loci');
PwrExEB(handles,3)

% --- Executes on button press in pushbutton2.
function pushbutton2_Callback(~, ~, handles) %#ok<DEFNU>
global fname
fname='';
%reset all labels an panels after the STRINFO computations
set(handles.pushbutton1,'Visible','on')
set(handles.pushbutton2,'Visible','off')
set(handles.Filemenu,'Enable','on')
set(handles.Demomenu,'Enable','on')
set(handles.STRinfomenu,'Enable','on')
set(handles.Reportmenu3,'Visible','off')
set(handles.Populationpopupmenu,'Enable','on')
set(handles.Kitpopupmenu,'Enable','on')
set(handles.Parentagemenu,'Enable','on')
Directparentage_Callback([],[],handles)
set(handles.uipanel6,'Title','Paternity Index','Visible','off')
set(handles.uipanel7,'Title','Random Man not Excluded','Visible','off')
for J=1:20
    eval(['set(handles.PIlabel' int2str(J) ',''String'','''',''Visible'',''on'')'])
    eval(['set(handles.RMNElabel' int2str(J) ',''String'','''',''Visible'',''on'')'])
    if (J>=1 && J<=3) || (J==8 || J==9)
        eval(['set(handles.uipanel' int2str(J) ',''Visible'',''on'')'])
    end
    if J<19
        eval(['cs=get(handles.text' int2str(J) ',''Position'');'])
        cs(1)=0.02; %#ok<NASGU>
        eval(['set(handles.text' int2str(J) ',''Position'',cs)'])
    end
end

%THESE ARE THE REPORT MENUS
% --------------------------------------------------------------------
function Reportmenu1_Callback(~, ~, handles) %#ok<DEFNU>
global CODISRootFolder fname 
%this function is recalled to generate an html report when we are not in the
%presence of an incestuous relationship
set(handles.Reportmenu1,'Visible','off') %turn off the menu
global fid PI RMNE MP CPI EP CMP FHp M
Rb2=isequal(get(handles.Directparentage,'Checked'),'on');
MFlag=isequal(M,zeros(size(M)));
numlines=1; name='Report Informations'; 
options.Resize='on'; options.WindowStyle='normal'; options.Interpreter='tex';
pathname=strcat(CODISRootFolder,'reports',filesep);
%Set the inputdlg options structure
defaultans={fname};
fname=char(inputdlg('Input the file name','Filename',1,defaultans,options));
cd(pathname)
if ~isempty(fname)
    filename=sprintf('%s.html',fname);
else
    filename='report.html';
end
fid = fopen(filename,'w+');
fprintf(fid,'<HTML><HEAD><TITLE>COmbined DNA Index System Report</TITLE></HEAD>\n');
fprintf(fid,'<BODY style="font-face:Arial;font-size=14px">\n');
fprintf(fid,'<H1 style="text-align:center;font-weight:bold;">CODIS Report</H1><BR><BR>\n');
x=get(handles.Populationpopupmenu,'Value');
pop=get(handles.Populationpopupmenu,'String');
fprintf(fid,'<P>Population: '); ReportTableRow(pop(x,:)); 
x=get(handles.Kitpopupmenu,'Value');
pop=get(handles.Kitpopupmenu,'String');
fprintf(fid,'<BR>Used kit: '); ReportTableRow(pop(x,:)); fprintf(fid,'</P>\n');
fprintf(fid,'<P>Incestuous relationship: no<BR>\n');
if Rb2==1
    fprintf(fid,'<P>Parentage: direct<BR>\n');
else
    fprintf(fid,'<P>Parentage: reverse<BR>\n');
end
if Rb2==1
    fprintf(fid,'Hp: Proband is paternally related to child as: ');
    x=get(handles.Hppopupmenu,'Value');
    pop=get(handles.Hppopupmenu,'String');
    ReportTableRow(pop(x,:)); fprintf(fid,'<BR>\n');
else
    fprintf(fid,'Hp: The alleged couple are the parents of the child<BR>\n');
end
if Rb2==1
    fprintf(fid,'Hd: Proband is paternally related to child as: ');
    x=get(handles.Hdpopupmenu,'Value');
    pop=get(handles.Hdpopupmenu,'String');
    ReportTableRow(pop(x,:)); fprintf(fid,'<BR>\n');
else
    fprintf(fid,'Hp: The alleged couple and the child are unrelated');
end
fprintf(fid,'</P><BR>\n');

fprintf(fid,'<TABLE BORDER="2" CELLPADDING="4" CELLSPACING="0" style="text-align:center;">\n');
fprintf(fid,'<TR><TH>STR</TH>');
if ~MFlag
    prompt={'Enter the mother name'};
    defaultanswer={'Mother'};
    answer=char(inputdlg(prompt,name,numlines,defaultanswer,options));
    fprintf(fid,'<TH COLSPAN=2>%s</TH>',answer);
end
prompt={'Enter the child name'};
defaultanswer={'Child'};
childname=char(inputdlg(prompt,name,numlines,defaultanswer,options));
fprintf(fid,'<TH COLSPAN=2>%s</TH>',childname);
str=get(handles.uipanel3,'Title');
prompt={'Enter the proband name'};
defaultanswer={str};
afname=char(inputdlg(prompt,name,numlines,defaultanswer,options));
fprintf(fid,'<TH COLSPAN=2>%s</TH>',afname);
fprintf(fid,'<TH>Comment</TH>');

if Rb2==1
    if FHp==1
        str='Paternity Index';
    elseif FHp==2 || FHp==3
        str='Sibship Index';
    else
        str='Avuncular Index';
    end
else
    str='Parentage Index';
end
fprintf(fid,'<TH>%s</TH></TR>\n',str);

for J=1:18
    eval(['strv=get(handles.text' int2str(J) ',''Visible'');']);
    if ~isempty(strfind(strv,'on')) %If there is the marker
        eval(['str=char(get(handles.text' int2str(J) ',''String''));']);
        fprintf(fid,'<TR><TD>%s</TD>\n',str);
        
        %The mother
        if ~MFlag
            eval(['l1=get(handles.Motherlabel' int2str(J) '_1,''String'');'])
            if ~isempty(l1)
                eval(['bgcolor=get(handles.Motherlabel' int2str(J) '_1,''TooltipString'');'])
                eval(['fontcolor=Colorname(ceil(255.*get(handles.Motherlabel' int2str(J) '_1,''ForegroundColor'')));'])
                eval(['fontweight=get(handles.Motherlabel' int2str(J) '_1,''FontWeight'');'])
                eval(['strv=get(handles.Motherlabel' int2str(J) '_2,''Visible'');']);
                if ~isempty(strfind(strv,'on'))            
                    cs=1;
                else
                    cs=2;
                end
                fprintf(fid,'<TD COLSPAN=%i style="background-color:%s;"><P style="color:%s;font-weight:%s;">',cs,bgcolor,fontcolor,fontweight); %#ok<NODEF>
                ReportTableRow(l1); fprintf(fid,'</P></TD>');
                if cs==1
                    eval(['l2=get(handles.Motherlabel' int2str(J) '_2,''String'');'])
                    eval(['bgcolor=get(handles.Motherlabel' int2str(J) '_2,''TooltipString'');'])
                    eval(['fontcolor=Colorname(ceil(255.*get(handles.Motherlabel' int2str(J) '_2,''ForegroundColor'')));'])
                    eval(['fontweight=get(handles.Motherlabel' int2str(J) '_2,''FontWeight'');'])
                    fprintf(fid,'<TD COLSPAN=%i style="background-color:%s;"><P style="color:%s;font-weight:%s;">',cs,bgcolor,fontcolor,fontweight);
                    ReportTableRow(l2); fprintf(fid,'</P></TD>');
                end
            else
                fprintf(fid,'<TD COLSPAN=2><P style="color:white;">.</P></TD>\n');
            end
        end
        
        %The child
        eval(['l1=get(handles.Childlabel' int2str(J) '_1,''String'');'])
        eval(['bgcolor=get(handles.Childlabel' int2str(J) '_1,''TooltipString'');'])
        eval(['fontcolor=Colorname(ceil(255.*get(handles.Childlabel' int2str(J) '_1,''ForegroundColor'')));'])
        eval(['fontweight=get(handles.Childlabel' int2str(J) '_1,''FontWeight'');'])
        eval(['strv=get(handles.Childlabel' int2str(J) '_2,''Visible'');']);
        if ~isempty(strfind(strv,'on'))
            cs=1;
        else
            cs=2;
        end
        fprintf(fid,'<TD COLSPAN=%i style="background-color:%s;"><P style="color:%s;font-weight:%s;">',cs,bgcolor,fontcolor,fontweight);
        ReportTableRow(l1); fprintf(fid,'</P></TD>');
        if cs==1
            eval(['l2=get(handles.Childlabel' int2str(J) '_2,''String'');'])
            eval(['bgcolor=get(handles.Childlabel' int2str(J) '_2,''TooltipString'');'])
            eval(['fontcolor=Colorname(ceil(255.*get(handles.Childlabel' int2str(J) '_2,''ForegroundColor'')));'])
            eval(['fontweight=get(handles.Childlabel' int2str(J) '_2,''FontWeight'');'])
            fprintf(fid,'<TD COLSPAN=%i style="background-color:%s;"><P style="color:%s;font-weight:%s;">',cs,bgcolor,fontcolor,fontweight);
            ReportTableRow(l2); fprintf(fid,'</P></TD>');
        end
        
        %The alleged relative
        eval(['l1=get(handles.Fatherlabel' int2str(J) '_1,''String'');'])
        eval(['bgcolor=get(handles.Fatherlabel' int2str(J) '_1,''TooltipString'');'])
        eval(['fontcolor=Colorname(ceil(255.*get(handles.Fatherlabel' int2str(J) '_1,''ForegroundColor'')));'])
        eval(['fontweight=get(handles.Fatherlabel' int2str(J) '_1,''FontWeight'');'])
        eval(['strv=get(handles.Fatherlabel' int2str(J) '_2,''Visible'');']);
        if ~isempty(strfind(strv,'on'))
            cs=1;
        else
            cs=2;
        end
        fprintf(fid,'<TD COLSPAN=%i style="background-color:%s;"><P style="color:%s;font-weight:%s;">',cs,bgcolor,fontcolor,fontweight);
        ReportTableRow(l1); fprintf(fid,'</P></TD>');
        if cs==1
            eval(['l2=get(handles.Fatherlabel' int2str(J) '_2,''String'');'])
            eval(['bgcolor=get(handles.Fatherlabel' int2str(J) '_2,''TooltipString'');'])
            eval(['fontcolor=Colorname(ceil(255.*get(handles.Fatherlabel' int2str(J) '_2,''ForegroundColor'')));'])
            eval(['fontweight=get(handles.Fatherlabel' int2str(J) '_2,''FontWeight'');'])
            fprintf(fid,'<TD COLSPAN=%i style="background-color:%s;"><P style="color:%s;font-weight:%s;">',cs,bgcolor,fontcolor,fontweight);
            ReportTableRow(l2); fprintf(fid,'</P></TD>');
        end
        
        %The comment
        eval(['bgcolor=get(handles.Commlabel' int2str(J) ',''TooltipString'');'])
        eval(['fontcolor=Colorname(ceil(255.*get(handles.Commlabel' int2str(J) ',''ForegroundColor'')));'])
        eval(['str=get(handles.Commlabel' int2str(J) ',''String'');']);
        fprintf(fid,'<TD style="background-color:%s;"><P style="color:%s;">%s</P></TD>',bgcolor,fontcolor,str);
        
        %The likelihood ratio
        fprintf(fid,'<TD>%0.8f</TD></TR>\n',PI(J));
    end
end
if ~MFlag
    fprintf(fid,'<TR><TH COLSPAN=8>CUMULATIVE</TH>');
else
    fprintf(fid,'<TR><TH COLSPAN=6>CUMULATIVE</TH>');
end


if CPI>1
    fprintf(fid,'<TH style="background-color:Darkorange;">%0.2f</TH></TR>\n',CPI);
else
    fprintf(fid,'<TH style="background-color:Darkorange;">%1.4e</TH></TR>\n',CPI);
end
fprintf(fid,'</TABLE>\n');

fontcolor=Colorname(ceil(255.*get(handles.text129,'ForegroundColor')));
str=get(handles.text129,'String');
fprintf(fid,'<P style="font-size:200%%;color:%s;">%s</P>\n',fontcolor,str);
fprintf(fid,'<br clear=all style="page-break-before:always">\n');

L=[5 10 20 30 40 50 60 70 80 90 95];
PrO=L./fliplr(L);
PsO=(1./(1+1./(PrO.*CPI))).*100;
fprintf(fid,'<table style="text-align: left;" border="1" cellpadding="1" cellspacing="1">\n');
fprintf(fid,'<tbody><tr><td style="text-align: center; background-color: Lightpink;">Hp Prob.</td>\n');
for I=1:5
    fprintf(fid,'<td style="text-align: center; background-color: Lightpink;">%i%%</td>\n',L(I));
end
fprintf(fid,'</tr><tr><td style="text-align: center; background-color: DeepSkyBlue;">Hd Prob.</td>\n');
for I=1:5
    fprintf(fid,'<td style="text-align: center; background-color: DeepSkyBlue">%i%%</td>\n',L(12-I));
end
fprintf(fid,'</tr><tr><td style="text-align: center; background-color: rgb(102, 255, 153);">Prior odds</td>\n');
for I=1:5
    fprintf(fid,'<td style="text-align: center; background-color: rgb(102, 255, 153);">%0.2f</td>\n',PrO(I));
end
fprintf(fid,'</tr><tr><td style="text-align: center;">Posterior Prob.</td>\n');
for I=1:5
    if PsO(I)>1
        fprintf(fid,'<td style="text-align: center;">%0.9f%%</td>\n',PsO(I));
    else
        fprintf(fid,'<td style="text-align: center;">1:%i</td>\n',1/PsO(I));
    end
end
fprintf(fid,'</tr><tr><td colspan="6" rowspan="1" style="text-align: center; background-color: Silver;">.</td></tr>\n');
fprintf(fid,'<tr><td style="text-align: center; background-color: Lightpink;">Hp Prob.</td>\n');
fprintf(fid,'<td colspan="5" rowspan="1" style="text-align: center; background-color: Lightpink;">50%%</td></tr>\n');
fprintf(fid,'<tr><td style="text-align: center; background-color: DeepSkyBlue;">Hd Prob.</td>\n');
fprintf(fid,'<td colspan="5" rowspan="1" style="text-align: center; background-color: DeepSkyBlue;">50%%</td></tr>\n');
fprintf(fid,'<tr><td style="vertical-align: top; text-align: center; background-color: rgb(102, 255, 153);">Prior Odds<br></td>\n');
fprintf(fid,'<td colspan="5" rowspan="1" style="vertical-align: top; text-align: center; background-color: rgb(102, 255, 153);">1<br></td></tr>\n');
fprintf(fid,'<tr><td style="text-align: center;">Posterior Prob.</td>\n');
if PsO(6)>1
    fprintf(fid,'<td colspan="5" rowspan="1" style="text-align: center; background-color: yellow;">%0.9f%%</td>\n',PsO(6));
else
    fprintf(fid,'<td colspan="5" rowspan="1" style="text-align: center; background-color: yellow;">1:%i</td>\n',1/PsO(6));
end
fprintf(fid,'</tr><tr><td colspan="6" rowspan="1" style="text-align: center; background-color: Silver;">.</td></tr>\n');
fprintf(fid,'<tr><td style="text-align: center; background-color: Lightpink;">Hp Prob.</td>\n');
for I=7:11
    fprintf(fid,'<td style="text-align: center; background-color: Lightpink;">%i%%</td>\n',L(I));
end
fprintf(fid,'</tr><tr><td style="text-align: center; background-color: DeepSkyBlue;">Hd Prob.</td>\n');
for I=7:11
    fprintf(fid,'<td style="text-align: center; background-color: DeepSkyBlue;">%i%%</td>\n',L(12-I));
end
fprintf(fid,'</tr><tr><td style="text-align: center; background-color: rgb(102, 255, 153);">Prior odds</td>\n');
for I=7:11
    fprintf(fid,'<td style="text-align: center; background-color: rgb(102, 255, 153);">%0.2f</td>\n',PrO(I));
end
fprintf(fid,'</tr><tr><td style="text-align: center;">Posterior Prob.</td>\n');
for I=7:11
    if PsO(I)>1
        fprintf(fid,'<td style="text-align: center;">%0.9f%%</td>\n',PsO(I));
    else
        fprintf(fid,'<td style="text-align: center;">1:%i</td>\n',1/PsO(I));
    end
end
fprintf(fid,'</tr></tbody></table><br>\n');


if Rb2==1 && (FHp~=2 && FHp~=3) 
     fprintf(fid,'<TABLE BORDER="2" CELLPADDING="4" CELLSPACING="0" style="text-align:center;">\n');
     fprintf(fid,'<TR><TH>STR</TH>');
     fprintf(fid,'<TH COLSPAN=2>%s</TH>',childname);
     fprintf(fid,'<TH>RMNE</TH>');
     str=get(handles.uipanel3,'Title');
     fprintf(fid,'<TH COLSPAN=2>%s</TH>',afname);
     fprintf(fid,'<TH>MP</TH></TR>\n');
     for J=1:18
         eval(['strv=get(handles.text' int2str(J) ',''Visible'');']);
         if ~isempty(strfind(strv,'on')) %If there is the marker
             eval(['str=char(get(handles.text' int2str(J) ',''String''));']);
             fprintf(fid,'<TR><TD><P>%s</P></TD>\n',str);
             
             %The child
             eval(['l1=get(handles.Childlabel' int2str(J) '_1,''String'');'])
             eval(['bgcolor=get(handles.Childlabel' int2str(J) '_1,''TooltipString'');'])
             eval(['fontcolor=Colorname(ceil(255.*get(handles.Childlabel' int2str(J) '_1,''ForegroundColor'')));'])
             eval(['fontweight=get(handles.Childlabel' int2str(J) '_1,''FontWeight'');'])
             eval(['strv=get(handles.Childlabel' int2str(J) '_2,''Visible'');']);
             if ~isempty(strfind(strv,'on'))
                 cs=1;
             else
                 cs=2;
             end
             fprintf(fid,'<TD COLSPAN=%i style="background-color:%s;"><P style="color:%s;font-weight:%s;">',cs,bgcolor,fontcolor,fontweight);
             ReportTableRow(l1); fprintf(fid,'</P></TD>');
             if cs==1
                 eval(['l2=get(handles.Childlabel' int2str(J) '_2,''String'');'])
                 eval(['bgcolor=get(handles.Childlabel' int2str(J) '_2,''TooltipString'');'])
                 eval(['fontcolor=Colorname(ceil(255.*get(handles.Childlabel' int2str(J) '_2,''ForegroundColor'')));'])
                 eval(['fontweight=get(handles.Childlabel' int2str(J) '_2,''FontWeight'');'])
                 fprintf(fid,'<TD COLSPAN=%i style="background-color:%s;"><P style="color:%s;font-weight:%s;">',cs,bgcolor,fontcolor,fontweight);
                 ReportTableRow(l2); fprintf(fid,'</P></TD>');
             end
             %THE RANDOM MAN NOT EXCLUDED
             fprintf(fid,'<TD>%0.4f</TD>',EP(J));
             
             %The alleged relative
             eval(['l1=get(handles.Fatherlabel' int2str(J) '_1,''String'');'])
             eval(['bgcolor=get(handles.Fatherlabel' int2str(J) '_1,''TooltipString'');'])
             eval(['fontcolor=Colorname(ceil(255.*get(handles.Fatherlabel' int2str(J) '_1,''ForegroundColor'')));'])
             eval(['fontweight=get(handles.Fatherlabel' int2str(J) '_1,''FontWeight'');'])
             eval(['strv=get(handles.Fatherlabel' int2str(J) '_2,''Visible'');']);
             if ~isempty(strfind(strv,'on'))
                 cs=1;
             else
                 cs=2;
             end
             fprintf(fid,'<TD COLSPAN=%i style="background-color:%s;"><P style="color:%s;font-weight:%s;">',cs,bgcolor,fontcolor,fontweight);
             ReportTableRow(l1); fprintf(fid,'</P></TD>');
             if cs==1
                 eval(['l2=get(handles.Fatherlabel' int2str(J) '_2,''String'');'])
                 eval(['bgcolor=get(handles.Fatherlabel' int2str(J) '_2,''TooltipString'');'])
                 eval(['fontcolor=Colorname(ceil(255.*get(handles.Fatherlabel' int2str(J) '_2,''ForegroundColor'')));'])
                 eval(['fontweight=get(handles.Fatherlabel' int2str(J) '_2,''FontWeight'');'])
                 fprintf(fid,'<TD COLSPAN=%i style="background-color:%s;"><P style="color:%s;font-weight:%s;">',cs,bgcolor,fontcolor,fontweight);
                 ReportTableRow(l2); fprintf(fid,'</P></TD>');
             end
             %The matching probability
             fprintf(fid,'<TD>%0.4f</TD></TR>\n',MP(J));
         end
     end

     fprintf(fid,'<TR><TH>CUMULATIVE</TH>');
     fprintf(fid,'<TH COLSPAN=2 style="background-color:Silver;"></TH><TH style="background-color:Darkorange;">%1.4e</TH>',RMNE);
     fprintf(fid,'<TH COLSPAN=2 style="background-color:Silver;"></TH><TH style="background-color:Darkorange;">%1.4e</TH></TR>\n',CMP);
     fprintf(fid,'<TR><TH>PROBABILITY</TH>');
     fprintf(fid,'<TH COLSPAN=2 style="background-color:Silver;"></TH><TH style="background-color:Darkorange;">1:%0.0f</TH>',1/RMNE);
     fprintf(fid,'<TH COLSPAN=2 style="background-color:Silver;"></TH><TH style="background-color:Darkorange;">1:%0.0f</TH></TR>',1/CMP);
     fprintf(fid,'</TABLE>\n');
end
fprintf(fid,'</BODY></HTML>\n');
fclose(fid);
cd(CODISRootFolder)

function Reportmenu2_Callback(~, ~, handles) %#ok<DEFNU>
%this function is recalled to generate an html report when we are in the
%presence of an incestuous relationship
set(handles.Reportmenu2,'Visible','off')
global fname CODISRootFolder ED WE fid 

numlines=1; name='Report Informations'; 
options.Resize='on'; options.WindowStyle='normal'; options.Interpreter='tex';
pathname=strcat(CODISRootFolder,'reports',filesep);
%Set the inputdlg options structure
defaultans={fname};
fname=inputdlg('Input the file name','Filename',1,defaultans,options);
cd(pathname)
if ~isempty(fname)
    filename=sprintf('%s.html',fname);
else
    filename='report.html';
end
fid = fopen(filename,'w+');

fprintf(fid,'<HTML><HEAD><TITLE>COmbined DNA Index System Report</TITLE></HEAD>\n');
fprintf(fid,'<BODY style="font-face:Arial;font-size=14px">\n');
fprintf(fid,'<H1 style="text-align:center;font-weight:bold;">CODIS Report</H1><BR><BR>\n');

x=get(handles.Populationpopupmenu,'Value');
pop=get(handles.Populationpopupmenu,'String');
fprintf(fid,'<P>Population: '); ReportTableRow(pop(x,:)); 
x=get(handles.Kitpopupmenu,'Value');
pop=get(handles.Kitpopupmenu,'String');
fprintf(fid,'<BR>Used kit: '); ReportTableRow(pop(x,:)); fprintf(fid,'</P>\n');
fprintf(fid,'<P>Incestuous relationship: yes<BR>\n');
fprintf(fid,'Type of incestuous relationship proposed: ');
x=get(handles.Hppopupmenu,'Value');
pop=get(handles.Hppopupmenu,'String');
ReportTableRow(pop(x,:));
fprintf(fid,'</P><BR>\n');

fprintf(fid,'<TABLE BORDER="2" CELLPADDING="4" CELLSPACING="0" style="text-align:center;">\n');
fprintf(fid,'<TR><TH>STR</TH>');
prompt={'Enter the mother name'};
defaultanswer={'Mother'};
answer=char(inputdlg(prompt,name,numlines,defaultanswer,options));
fprintf(fid,'<TH COLSPAN=2>%s</TH>',answer);
prompt={'Enter the proband name'};
defaultanswer={'Child'};
answer=char(inputdlg(prompt,name,numlines,defaultanswer,options));
fprintf(fid,'<TH COLSPAN=2>%s</TH>',answer);
fprintf(fid,'<TH>Incest Index by Edwards</TH><TH>Incest Index by Wenk</TH></TR>\n');

for J=1:18
    eval(['strv=get(handles.text' int2str(J) ',''Visible'');']);
    if ~isempty(strfind(strv,'on')) %If there is the marker
        eval(['str=get(handles.text' int2str(J) ',''String'');']);
        fprintf(fid,'<TR><TD>%s</TD>\n',str); %#ok<NODEF>
        
        %The mother
        eval(['l1=get(handles.Motherlabel' int2str(J) '_1,''String'');'])
        eval(['bgcolor=get(handles.Motherlabel' int2str(J) '_1,''TooltipString'');'])
        eval(['fontcolor=Colorname(ceil(255.*get(handles.Motherlabel' int2str(J) '_1,''ForegroundColor'')));'])
        eval(['fontweight=get(handles.Motherlabel' int2str(J) '_1,''FontWeight'');'])
        eval(['strv=get(handles.Motherlabel' int2str(J) '_2,''Visible'');']);
        if ~isempty(strfind(strv,'on'))
            cs=1;
        else
            cs=2;
        end
        fprintf(fid,'<TD COLSPAN=%i style="background-color:%s;"><P style="color:%s;font-weight:%s;">',cs,bgcolor,fontcolor,fontweight);  %#ok<NODEF>
        ReportTableRow(l1); fprintf(fid,'</P></TD>');
        if cs==1
            eval(['l2=get(handles.Motherlabel' int2str(J) '_2,''String'');'])
            eval(['bgcolor=get(handles.Motherlabel' int2str(J) '_2,''TooltipString'');'])
            eval(['fontcolor=Colorname(ceil(255.*get(handles.Motherlabel' int2str(J) '_2,''ForegroundColor'')));'])
            eval(['fontweight=get(handles.Motherlabel' int2str(J) '_2,''FontWeight'');'])
            fprintf(fid,'<TD COLSPAN=%i style="background-color:%s;"><P style="color:%s;font-weight:%s;">',cs,bgcolor,fontcolor,fontweight);
            ReportTableRow(l2); fprintf(fid,'</P></TD>');
        end
        
        %The child
        eval(['l1=get(handles.Childlabel' int2str(J) '_1,''String'');'])
        eval(['bgcolor=get(handles.Childlabel' int2str(J) '_1,''TooltipString'');'])
        eval(['fontcolor=Colorname(ceil(255.*get(handles.Childlabel' int2str(J) '_1,''ForegroundColor'')));'])
        eval(['fontweight=get(handles.Childlabel' int2str(J) '_1,''FontWeight'');'])
        eval(['strv=get(handles.Childlabel' int2str(J) '_2,''Visible'');']);
        if ~isempty(strfind(strv,'on'))
            cs=1;
        else
            cs=2;
        end
        fprintf(fid,'<TD COLSPAN=%i style="background-color:%s;"><P style="color:%s;font-weight:%s;">',cs,bgcolor,fontcolor,fontweight);
        ReportTableRow(l1); fprintf(fid,'</P></TD>');
        if cs==1
            eval(['l2=get(handles.Childlabel' int2str(J) '_2,''String'');'])
            eval(['bgcolor=get(handles.Childlabel' int2str(J) '_2,''TooltipString'');'])
            eval(['fontcolor=Colorname(ceil(255.*get(handles.Childlabel' int2str(J) '_2,''ForegroundColor'')));'])
            eval(['fontweight=get(handles.Childlabel' int2str(J) '_2,''FontWeight'');'])
            fprintf(fid,'<TD COLSPAN=%i style="background-color:%s;"><P style="color:%s;font-weight:%s;">',cs,bgcolor,fontcolor,fontweight);
            ReportTableRow(l2); fprintf(fid,'</P></TD>');
        end
        
        %The indices
        fprintf(fid,'<TD>%0.8f</TD>',ED(J));
        fprintf(fid,'<TD>%0.8f</TD>',WE(J));
        fprintf(fid,'</TR>\n');
    end
end
b=prod(WE);

fprintf(fid,'<TR><TH COLSPAN=5>CUMULATIVE</TH>');
if ED(19)>1
    fprintf(fid,'<TH style="background-color:Darkorange;">%0.2f</TH>\n',ED(19));
else
    fprintf(fid,'<TH style="background-color:Darkorange;">%1.4e</TH>\n',ED(19));
end

if b>1
    fprintf(fid,'<TH style="background-color:Darkorange;">%0.2f</TH></TR>\n',b);
else
    fprintf(fid,'<TH style="background-color:Darkorange;">%1.4e</TH></TR>\n',b);
end

fprintf(fid,'<TR><TH COLSPAN=5>PROBABILITY</TH>');
if ED(19)>1
    fprintf(fid,'<TH style="background-color:Darkorange;">%0.15f%%</TH>\n',100*(1/(1+1/ED(19))));
else
    fprintf(fid,'<TH style="background-color:Darkorange;">1:%0.0f</TH>\n',round(1/ED(19)));
end
if b>1
    fprintf(fid,'<TH style="background-color:Darkorange;">%0.15f%%</TH></TR>\n',100*(1/(1+1/b)));
else
    fprintf(fid,'<TH style="background-color:Darkorange;">1:%0.0f</TH></TR>\n',round(1/b));
end

fprintf(fid,'<TR><TH COLSPAN=5>COMMENT</TH>');
fontcolor=Colorname(ceil(255.*get(handles.textED,'ForegroundColor')));
str=get(handles.textED,'String');
fprintf(fid,'<TD><P style="color:%s;font-weight:bold;">%s</P></TD>',fontcolor,str);
fontcolor=Colorname(ceil(255.*get(handles.textWE,'ForegroundColor')));
str=get(handles.textWE,'String');
fprintf(fid,'<TD><P style="color:%s;font-weight:bold;">%s</P></TD>',fontcolor,str);
fprintf(fid,'</TABLE>\n');
fontcolor=Colorname(ceil(255.*get(handles.text129,'ForegroundColor')));
str=get(handles.text129,'String');
fprintf(fid,'<P style="font-size:200%%;color:%s">%s</P>\n',fontcolor,str);
fprintf(fid,'</BODY></HTML>\n');
fclose(fid);

function Reportmenu3_Callback(~, ~, handles) %#ok<DEFNU>
%this function is recalled to generate an html report for STRinfomenu
global fid STRstr1 STRstr2 CODISRootFolder 
set(handles.Reportmenu3,'Visible','off')
pathname=strcat(CODISRootFolder,'stats',filesep);
%Set the inputdlg options structure
cd(pathname)
title=strcat(STRstr2,'_report.html');
fid = fopen(title,'w+');
fprintf(fid,'<HTML><HEAD><TITLE>COmbined DNA Index System Report</TITLE></HEAD>\n');
fprintf(fid,'<BODY style="font-face:Arial;font-size=14px">\n');
fprintf(fid,'<H1 style="text-align:center;font-weight:bold;">CODIS Report</H1>\n');
fprintf(fid,'<H2 style="text-align:center;font-weight:bold;">%s</H2>\n',STRstr1);
fprintf(fid,'<H3 style="text-align:center;font-weight:bold;">%s</H3><BR><BR>\n',STRstr2);
x=get(handles.Populationpopupmenu,'Value');
pop=get(handles.Populationpopupmenu,'String');
fprintf(fid,'<P>Population: '); ReportTableRow(pop(x,:)); 
x=get(handles.Kitpopupmenu,'Value');
pop=get(handles.Kitpopupmenu,'String');
fprintf(fid,'<BR>Used kit: '); ReportTableRow(pop(x,:)); fprintf(fid,'</P><BR>\n');

fprintf(fid,'<TABLE BORDER="2" CELLPADDING="4" CELLSPACING="0" style="text-align:center;">\n');
fprintf(fid,'<TR><TH><P style="color:red;">STR</P></TH>');
str=get(handles.uipanel6,'Title');
fprintf(fid,'<TH>%s</TH>',str);
strx=get(handles.uipanel7,'Visible');
if ~isempty(strfind(strx,'on'))
    str=get(handles.uipanel7,'Title');
    fprintf(fid,'<TH>%s</TH>',str);
end
fprintf(fid,'</TR>\n');
for J=1:18
    eval(['strv=get(handles.text' int2str(J) ',''Visible'');']);
    if ~isempty(strfind(strv,'on')) %If there is the marker
        eval(['str=get(handles.text' int2str(J) ',''String'');']);
        fprintf(fid,'<TR><TD><P style="color:red;font-weight:bold;">%s</P></TD>\n',str);
        eval(['str=get(handles.PIlabel' int2str(J) ',''String'');'])
        fprintf(fid,'<TD>'); ReportTableRow(str); fprintf(fid,'</TD>');
        if ~isempty(strfind(strx,'on'))
            eval(['str=get(handles.RMNElabel' int2str(J) ',''String'');'])
            fprintf(fid,'<TD>'); ReportTableRow(str); fprintf(fid,'</TD>');
        end
        fprintf(fid,'</TR>\n');
    end
end
fprintf(fid,'<TR><TH>OVERALL</TH>');
str=get(handles.PIlabel20,'String');
fprintf(fid,'<TH style="background-color:Darkorange;">');
ReportTableRow(str); fprintf(fid,'</TH>');
if ~isempty(strfind(strx,'on'))
    str=get(handles.RMNElabel20,'String');
    fprintf(fid,'<TH style="background-color:Darkorange;">');
    ReportTableRow(str); fprintf(fid,'</TH>');
end
fprintf(fid,'</TR></TABLE></BODY></HTML>\n');
fclose(fid);
cd(CODISRootFolder)


%THESE ARE SEVERAL SUBROUTINES CALLED BY THIS GUI

function popdataset(handles)
%This function sets as visible the popupmenu of the STRs used for the selected
%kit. Even if a STR is determined in the used kit but the alleles frequencies
%are not available for the selected population, the popupmenu is switched off
%(i.e. in the ABI AmpFlSTR SEfiler the SE33 STR is amplified; however, the SE33
%alleles frequencies are unavailable for the italian population and so all SE33
%labels are switched off)
global KIT Kitsorted CODISpop NL
NL=length(CODISpop);%there are NL loci in the dataset
TF=sum(strcmp(Kitsorted(get(handles.Kitpopupmenu,'Value')),KIT.Kitname).*(1:1:length(KIT.Kitname))');
Idx=KIT.Kitindex(TF,:); loci=CODISpop(Idx,7); a=KIT.Kitdata(TF,:); a=a(Idx);
for J=1:18
    if a(J) && ~isempty(CODISpop{Idx(J),1})
        s='on'; %#ok<NASGU>
    else
        s='off'; %#ok<NASGU>
    end
    eval(['set(handles.text' int2str(J) ',''Visible'',s,''String'',loci(J))'])
    eval(['set(handles.Motherpopupmenu' int2str(J) '_1,''Visible'',s,''String'',CODISpop{Idx(J),6}'')'])
    eval(['set(handles.Motherpopupmenu' int2str(J) '_2,''Visible'',s,''String'',CODISpop{Idx(J),6}'')'])
    eval(['set(handles.Childpopupmenu' int2str(J) '_1,''Visible'',s,''String'',CODISpop{Idx(J),6}'')'])
    eval(['set(handles.Childpopupmenu' int2str(J) '_2,''Visible'',s,''String'',CODISpop{Idx(J),6}'')'])
    eval(['set(handles.Fatherpopupmenu' int2str(J) '_1,''Visible'',s,''String'',CODISpop{Idx(J),6}'')'])
    eval(['set(handles.Fatherpopupmenu' int2str(J) '_2,''Visible'',s,''String'',CODISpop{Idx(J),6}'')'])
end

function guiloadmenu(handles)
global M C F Pop Kit Rb1 Rb2 Kitsorted 
set(handles.loadmenu,'Enable','off')
set(handles.Savemenu,'Enable','on')
set(handles.Merge,'Enable','off')
set(handles.Optionsmenu,'Enable','on')
set(handles.Demomenu,'Enable','off')
set(handles.STRinfomenu,'Enable','off')
set(handles.Populationpopupmenu,'Value',Pop)
J=sum(strcmp(Kit,Kitsorted).*(1:1:length(Kitsorted))');
set(handles.Kitpopupmenu,'Value',J)
popdataset(handles)
if Rb1==1
    Incestyes_Callback([], [], handles)
else
    Incestno_Callback([], [], handles)
    if Rb2==1
        Directparentage_Callback([], [], handles)
    else
        Reverseparentage_Callback([], [], handles)
    end
end
for J=1:18
    if M(J,:)~=0
        eval(['set(handles.Motherpopupmenu' int2str(J) '_1,''Value'',M(' int2str(J) ',1))'])
        eval(['set(handles.Motherpopupmenu' int2str(J) '_2,''Value'',M(' int2str(J) ',2))'])
    else
        eval(['set(handles.Motherpopupmenu' int2str(J) '_1,''Value'',1)'])
        eval(['set(handles.Motherpopupmenu' int2str(J) '_2,''Value'',1)'])
    end
    if C(J,:)~=0
        eval(['set(handles.Childpopupmenu' int2str(J) '_1,''Value'',C(' int2str(J) ',1))'])
        eval(['set(handles.Childpopupmenu' int2str(J) '_2,''Value'',C(' int2str(J) ',2))'])
    else
        eval(['set(handles.Childpopupmenu' int2str(J) '_1,''Value'',1)'])
        eval(['set(handles.Childpopupmenu' int2str(J) '_2,''Value'',1)'])
    end
    if F(J,:)~=0
        eval(['set(handles.Fatherpopupmenu' int2str(J) '_1,''Value'',F(' int2str(J) ',1))'])
        eval(['set(handles.Fatherpopupmenu' int2str(J) '_2,''Value'',F(' int2str(J) ',2))'])
    else
        eval(['set(handles.Fatherpopupmenu' int2str(J) '_1,''Value'',1)'])
        eval(['set(handles.Fatherpopupmenu' int2str(J) '_2,''Value'',1)'])
    end
end

function resetgui(handles)
global M C F FHp FHd
M=ones(18,2); C=M; F=M; FHp=1; FHd=1;
set(handles.Filemenu,'Enable','on')
set(handles.loadmenu,'Enable','on')
set(handles.Savemenu,'Enable','off')
set(handles.Merge,'Enable','on')
set(handles.Optionsmenu,'Enable','on')
set(handles.Demomenu,'Enable','on')
set(handles.STRinfomenu,'Enable','on')
for J=1:18
    eval(['cs=get(handles.Motherlabel' int2str(J) '_1,''Position'');'])
    cs(3)=0.3; cs(1)=0.12;
    eval(['set(handles.Motherlabel' int2str(J) '_1,''Position'',cs,''BackgroundColor'',''white'',''TooltipString'',''White'',''ForegroundColor'',''black'',''FontWeight'',''normal'',''String'','''',''Visible'',''off'')'])
    eval(['set(handles.Childlabel' int2str(J) '_1,''Position'',cs,''BackgroundColor'',[1 0.6 0.784],''TooltipString'',''Lightpink'',''ForegroundColor'',''black'',''FontWeight'',''normal'',''String'','''',''Visible'',''off'')'])
    eval(['set(handles.Fatherlabel' int2str(J) '_1,''Position'',cs,''BackgroundColor'',[0.678 0.922 1],''TooltipString'',''DeepSkyBlue'',''ForegroundColor'',''black'',''FontWeight'',''normal'',''String'','''',''Visible'',''off'')'])
    cs(1)=0.57;
    eval(['set(handles.Motherlabel' int2str(J) '_2,''Position'',cs,''BackgroundColor'',[1 0.6 0.784],''TooltipString'',''Lightpink'',''ForegroundColor'',''black'',''FontWeight'',''normal'',''String'','''',''Visible'',''off'')'])
    eval(['set(handles.Childlabel' int2str(J) '_2,''Position'',cs,''BackgroundColor'',[0.678 0.922 1],''TooltipString'',''DeepSkyBlue'',''ForegroundColor'',''black'',''FontWeight'',''normal'',''String'','''',''Visible'',''off'')'])
    eval(['set(handles.Fatherlabel' int2str(J) '_2,''Position'',cs,''BackgroundColor'',''white'',''TooltipString'',''White'',''ForegroundColor'',''black'',''FontWeight'',''normal'',''String'','''',''Visible'',''off'')'])
    eval(['set(handles.Motherpopupmenu' int2str(J) '_1,''Value'',1,''Visible'',''on'')'])
    eval(['set(handles.Motherpopupmenu' int2str(J) '_2,''Value'',1,''Visible'',''on'')'])
    eval(['set(handles.Childpopupmenu' int2str(J) '_1,''Value'',1,''Visible'',''on'')'])
    eval(['set(handles.Childpopupmenu' int2str(J) '_2,''Value'',1,''Visible'',''on'')'])
    eval(['set(handles.Fatherpopupmenu' int2str(J) '_1,''Value'',1,''Visible'',''on'')'])
    eval(['set(handles.Fatherpopupmenu' int2str(J) '_2,''Value'',1,''Visible'',''on'')'])
    eval(['set(handles.Commlabel' int2str(J) ',''BackgroundColor'',''white'',''TooltipString'',''White'',''ForegroundColor'',''green'',''String'','''',''Visible'',''on'')'])
    eval(['set(handles.PIlabel' int2str(J) ',''String'','''',''Visible'',''on'')'])
    eval(['set(handles.RMNElabel' int2str(J) ',''String'','''',''Visible'',''on'')'])
    eval(['set(handles.text' int2str(J) ',''Visible'',''on'')'])
    if J>=5 && J<=7
        eval(['set(handles.uipanel' int2str(J) ',''Visible'',''off'')'])
    end
end
set(handles.uipanel1,'Title','Mother')
set(handles.uipanel2,'Title','Child')
set(handles.uipanel3,'Title','Proband')
set(handles.uipanel6,'Title','Paternity Index')
set(handles.PIlabel19,'String','')
set(handles.PIlabel20,'String','')
set(handles.RMNElabel19,'String','')
set(handles.RMNElabel20,'String','')
set(handles.text127,'Visible','off')
set(handles.text128,'String','','Visible','off')
set(handles.text129,'String','','Visible','off')
set(handles.textED,'String','','Visible','off')
set(handles.textWE,'String','','Visible','off')
set(handles.uipanel1,'BackgroundColor',[1 0.6 0.784])
set(handles.uipanel2,'BackgroundColor',[1 1 1])
set(handles.uipanel3,'BackgroundColor',[0.678 0.922 1])
set(handles.Populationpopupmenu,'Value',1)
set(handles.Kitpopupmenu,'Value',1)
popdataset(handles)

function STRinfoguiprep(handles)
set(handles.pushbutton1,'Visible','off')
set(handles.pushbutton2,'Visible','on')
set(handles.Filemenu,'Enable','off')
set(handles.Demomenu,'Enable','off')
set(handles.STRinfomenu,'Enable','off')
set(handles.Populationpopupmenu,'Enable','off')
set(handles.Kitpopupmenu,'Enable','off')
set(handles.Reportmenu3,'Visible','on')
set(handles.Parentagemenu,'Enable','off')
ar=[1 2 3 8 9];
for J=1:5
    eval(['set(handles.uipanel' int2str(ar(J)) ',''Visible'',''off'')'])
end

function ReportTableRow(s)
%This function is used to write in the report file a cell array. The cell array
%is not compatible with the fprintf function
global fid
if iscell(s)
    for K=1:size(s,1)
        fprintf(fid,s{K,:});
    end
else
    fprintf(fid,s);
end

function str=Colorname(a)
%this function is used to identify the color name to use in the html tags
color=[255 255 255; 0 0 0; 255 0 0; 0 255 0;];
C={'White';'Black';'Red';'Green'};
[~,LOC] = ismember(a,color,'rows');
str=C{LOC};

function PwrEx(handles,loci)
STRinfoguiprep(handles)
parameters(1,handles,6)
OPE(handles,loci)

function PwrExEB(handles,loci)
STRinfoguiprep(handles)
set(handles.uipanel6,'Title','PER Elder brother - with mother','Visible','on')
set(handles.PIlabel19,'String','Overall PE')
parameters(1,handles,7)
OPE(handles,loci)

function OPE(handles,loci)
global A
%show the results
OPE=1-prod(1-A(~isnan(A))); %Overall Power of exclusion
if loci~=0
    PPNE=A(~isnan(A)); %Positive PPNE
    cPPNE=1-PPNE; pPPNE=prod(cPPNE);
    CF=pPPNE*sum(PPNE./cPPNE);
    OPE=OPE-CF;
    if loci==3
        [Pi Pj]=ndgrid(PPNE,PPNE); CF=pPPNE*sum(sum(tril((Pi.*Pj)./((1-Pi).*(1-Pj)),-1)));
        OPE=OPE-CF;
    end
end
set(handles.PIlabel20,'String',sprintf('%0.12f%%',100*OPE))

function parameters(params,handles,formula) %#ok<INUSL>
global A B NL CODISpop motherflag phi
A=NaN(18,1); B=NaN(18,1); %vector preallocations
for J=1:18
    if eval(sprintf('strcmp(get(handles.text%i,''Visible''),''on'')',J))==1
        eval(sprintf('locus=get(handles.text%i,''String'');',J))
        STR=sum(strcmp(locus,CODISpop(:,7)).*(1:1:NL)');
        p=CODISpop{STR,1};
        switch formula
            case 1
                A(J)=sum(sum((p'*p).^2)); %Probability of Match
                B(J)=1-A(J); %Probability of Discrimination
            case 2
                p=sort(p); 
                p(p==0)=[]; %delete null alleles
                A(J)=2*p(end)*p(end-1); %Worst Probability of Match
                B(J)=p(1)^2; %Best Probability of Match
            case 3
                mfa=p'*p; %matrix of binomial expansion of allelic frequencies
                xpic=diag(mfa);
                mpic=xpic*xpic';
                mpic=tril(mpic)+triu(mpic,1)'; %triangularize the matrix
                A(J)=1-sum(xpic)-sum(sum(tril(mpic,-1))); %Polymorphism Information Content
            case 4
                A(J)=1/(2*sum(p.^2)); %Typical Paternity Index
            case 5
                [N,M]=ndgrid(p,p);
                A(J)=sum(p.^3.*(1-p).^2)+sum(p.*(1-p).^3)+sum(sum(tril(N.*M.*(N+M).*(1-N-M).^2,-1))); %Mean exclusion chance
            case 6
                %This function computes the power of exclusion when mother is available
                %(flag==1) and when mother is unavailable (flag==-1). 
                %Phi is related to the kinship coefficient
                n=length(p);
                x=sum(p.*(1-p).^2);
                y=zeros(1,n-1);
                for I=1:n-1
                    if motherflag==1 %with mother
                        y(I)=sum(p(I)^2.*p(I+1:n).^2.*(4-3.*p(I)-3.*p(I+1:n)));
                    elseif motherflag==-1 %without mother
                        y(I)=sum(2.*p(I).*p(I+1:n).*(1-p(I)-p(I+1:n)).^2);
                    end
                end
                A(J)=phi*(x+motherflag*sum(y)); %Power of exclusion mother unrelated
            case 7
                A(J)=0.5-5/4*sum(p.^2)+sum(p.^3)-1/4*sum(p.^4); %Power of exclusion mother related
        end
        eval(['cs=get(handles.text' int2str(J) ',''Position'');'])
        cs(1)=0.63; %#ok<NASGU>
        eval(['set(handles.text' int2str(J) ',''Position'',cs)'])
        eval(['set(handles.PIlabel' int2str(J) ',''String'',A(J))'])
        if params==2
            eval(['set(handles.RMNElabel' int2str(J) ',''String'',B(J))'])
        end
    else
        eval(['set(handles.PIlabel' int2str(J) ',''Visible'',''off'')'])
        eval(['set(handles.RMNElabel' int2str(J) ',''Visible'',''off'')'])
    end
end
